# -*- coding: utf-8 -*-

from __future__ import unicode_literals

try:
    from reversion.revisions import create_revision, get_for_object
except ImportError:
    # django-reversion < 1.9
    from reversion import create_revision, get_for_object
import six

from django.db import transaction
from parler.utils.context import switch_language

from aldryn_reversion.core import create_revision as aldryn_create_revision

from ..models import JobCategory, JobOpening, JobApplication
from .base import JobsBaseTestCase


class ReversionTestCase(JobsBaseTestCase):
    """
    Cover reversion support, test cases:
     * revisions are created (reversions and self test)
     * revisions are reverted (reversions and self test)
     * not translated fields are changed when new revision is created
     * not translated fields are being reverted correctly
     * translated fields are changed on save with revision (latest change
       should be shown)
     * translated fields are reverted correctly
     * placeholder fields are reverted with correct plugins (according
       to revision)
     * [TBD] placeholder fields for translated objects should serve correct
       plugins for translation
     * [TBD] placeholder fields for translated objects should server correct
       plugins for translation after revision revert.
    """

    def create_revision(self, obj, content=None, **kwargs):
        with transaction.atomic():
            with create_revision():
                # populate event with new values
                for property, value in six.iteritems(kwargs):
                    setattr(obj, property, value)
                if content:
                    # get correct plugin for language. do not update the same
                    # one.
                    language = obj.get_current_language()
                    plugins = obj.content.get_plugins().filter(
                        language=language)
                    plugin = plugins[0].get_plugin_instance()[0]
                    plugin.body = content
                    plugin.save()
                obj.save()

    def revert_to(self, object_with_revision, revision_number):
        """
        Revert <object with revision> to revision number.
        """
        # get by position, since reversion_id is not reliable,
        version = list(reversed(
            get_for_object(
                object_with_revision)))[revision_number - 1]
        version.revision.revert()

    # Following tests does not covers translations!
    def test_revision_is_created_on_job_category_object_create(self):
        with transaction.atomic():
            with create_revision():
                job_category = JobCategory.objects.create(
                    app_config=self.app_config,
                    **self.category_values_raw['en'])
        self.assertEqual(len(get_for_object(job_category)), 1)

    def test_revision_is_created_on_job_opening_object_created(self):
        with transaction.atomic():
            with create_revision():
                job_opening = JobOpening.objects.create(
                    category=self.default_category,
                    **self.opening_values_raw['en'])
        self.assertEqual(len(get_for_object(job_opening)), 1)

    def test_category_revision_is_created(self):
        category = JobCategory.objects.create(
            app_config=self.app_config,
            **self.category_values_raw['en'])
        self.assertEqual(len(get_for_object(category)), 0)
        new_values_en_1 = self.make_new_values(
            self.category_values_raw['en'], 1)
        self.create_revision(category, **new_values_en_1)
        self.assertEqual(len(get_for_object(category)), 1)

    def test_job_opening_revision_is_created(self):
        job_opening = self.create_default_job_opening()
        self.assertEqual(len(get_for_object(job_opening)), 0)

        new_values_en_1 = self.make_new_values(
            self.default_job_values['en'], 1)
        self.create_revision(job_opening, **new_values_en_1)
        self.assertEqual(len(get_for_object(job_opening)), 1)

    def test_category_with_one_revision_contains_latest_values(self):
        category = JobCategory.objects.create(
            app_config=self.app_config,
            **self.category_values_raw['en'])
        new_values_en_1 = self.make_new_values(
            self.category_values_raw['en'], 1)
        # revision 1
        self.create_revision(category, **new_values_en_1)
        for prop in new_values_en_1.keys():
            self.assertEqual(getattr(category, prop), new_values_en_1[prop])

        # the same but with re-getting category object
        category = JobCategory.objects.get(pk=category.pk)
        for prop in new_values_en_1.keys():
            self.assertEqual(getattr(category, prop), new_values_en_1[prop])

    def test_opening_with_one_revision_contains_latest_values(self):
        job_opening = self.create_default_job_opening()
        # revision 1
        new_values_en_1 = self.make_new_values(self.opening_values_raw['en'], 1)
        content_en_1 = self.plugin_values_raw['en'].format(1)
        self.create_revision(
            job_opening, content=content_en_1, **new_values_en_1)

        job_opening = JobOpening.objects.get(pk=job_opening.pk)
        for prop in new_values_en_1.keys():
            self.assertEqual(getattr(job_opening, prop), new_values_en_1[prop])

    def test_opening_with_one_revision_serves_latest_content(self):
        job_opening = self.create_default_job_opening()
        # revision 1
        new_values_en_1 = self.make_new_values(self.opening_values_raw['en'], 1)
        content_en_1 = self.plugin_values_raw['en'].format(1)
        self.create_revision(
            job_opening, content=content_en_1, **new_values_en_1)

        # test url and served content
        with switch_language(job_opening, 'en'):
            url_1 = job_opening.get_absolute_url()
        response = self.client.get(url_1)
        self.assertContains(response, new_values_en_1['title'])
        self.assertContains(response, content_en_1)

    def test_category_with_two_revision_contains_latest_values(self):
        category = JobCategory.objects.create(
            app_config=self.app_config,
            **self.category_values_raw['en'])
        # revision 1
        new_values_en_1 = self.make_new_values(
            self.category_values_raw['en'], 1)
        self.create_revision(category, **new_values_en_1)

        # revision 2
        new_values_en_2 = self.make_new_values(
            self.category_values_raw['en'], 2)
        self.create_revision(category, **new_values_en_2)

        for prop in new_values_en_2.keys():
            self.assertEqual(getattr(category, prop), new_values_en_2[prop])

        # the same but with re-getting category object
        category = JobCategory.objects.get(pk=category.pk)
        for prop in new_values_en_2.keys():
            self.assertEqual(getattr(category, prop), new_values_en_2[prop])

    def test_opening_with_two_revision_contains_latest_values(self):
        job_opening = self.create_default_job_opening()

        # revision 1
        new_values_en_1 = self.make_new_values(self.opening_values_raw['en'], 1)
        content_en_1 = self.plugin_values_raw['en'].format(1)
        self.create_revision(
            job_opening, content=content_en_1, **new_values_en_1)

        # revision 2
        new_values_en_2 = self.make_new_values(self.opening_values_raw['en'], 2)
        content_en_2 = self.plugin_values_raw['en'].format(2)
        self.create_revision(
            job_opening, content=content_en_2, **new_values_en_2)

        job_opening = JobOpening.objects.get(pk=job_opening.pk)
        for prop in new_values_en_2.keys():
            self.assertEqual(getattr(job_opening, prop), new_values_en_2[prop])

    def test_opening_with_two_revision_serves_latest_content(self):
        job_opening = self.create_default_job_opening()

        # revision 1
        new_values_en_1 = self.make_new_values(self.opening_values_raw['en'], 1)
        content_en_1 = self.plugin_values_raw['en'].format(1)
        self.create_revision(
            job_opening, content=content_en_1, **new_values_en_1)

        # revision 2
        new_values_en_2 = self.make_new_values(self.opening_values_raw['en'], 2)
        content_en_2 = self.plugin_values_raw['en'].format(2)
        self.create_revision(
            job_opening, content=content_en_2, **new_values_en_2)

        # test served title
        job_opening = JobOpening.objects.get(pk=job_opening.pk)
        with switch_language(job_opening, 'en'):
            url = job_opening.get_absolute_url()
        response = self.client.get(url)
        self.assertContains(response, new_values_en_2['title'])
        self.assertContains(response, content_en_2)
        # legacy values/content
        self.assertNotContains(response, new_values_en_1['title'])
        self.assertNotContains(response, content_en_1)

    def test_category_revisions_are_reverted(self):
        # no revisions at this point
        category = JobCategory.objects.create(
            app_config=self.app_config,
            **self.category_values_raw['en'])

        # revision 1
        new_values_en_1 = self.make_new_values(
            self.category_values_raw['en'], 1)
        self.create_revision(category, **new_values_en_1)

        # revision 2
        new_values_en_2 = self.make_new_values(
            self.category_values_raw['en'], 2)
        self.create_revision(category, **new_values_en_2)

        # revert to 1
        self.revert_to(category, 1)

        category = JobCategory.objects.get(pk=category.pk)
        for prop in new_values_en_1.keys():
            self.assertEqual(getattr(category, prop), new_values_en_1[prop])

    def test_opening_revisions_are_reverted(self):
        job_opening = self.create_default_job_opening()

        # revision 1
        new_values_en_1 = self.make_new_values(self.opening_values_raw['en'], 1)
        content_en_1 = self.plugin_values_raw['en'].format(1)
        self.create_revision(
            job_opening, content=content_en_1, **new_values_en_1)

        # revision 2
        new_values_en_2 = self.make_new_values(self.opening_values_raw['en'], 2)
        content_en_2 = self.plugin_values_raw['en'].format(2)
        new_category = JobCategory.objects.create(
            app_config=self.app_config,
            **self.make_new_values(self.category_values_raw['en'], 2))
        new_values_en_2['category'] = new_category
        self.create_revision(
            job_opening, content=content_en_2, **new_values_en_2)

        # revert to 1
        self.revert_to(job_opening, 1)
        job_opening = JobOpening.objects.get(pk=job_opening.pk)
        new_values_en_1['category'] = self.default_category
        for prop in new_values_en_1.keys():
            self.assertEqual(getattr(job_opening, prop), new_values_en_1[prop])
        self.assertNotEqual(job_opening.category, new_category)

    def test_opening_reverted_revision_serves_appropriate_content(self):
        job_opening = self.create_default_job_opening()

        # revision 1
        new_values_en_1 = self.make_new_values(self.opening_values_raw['en'], 1)
        content_en_1 = self.plugin_values_raw['en'].format(1)
        self.create_revision(
            job_opening, content=content_en_1, **new_values_en_1)

        # revision 2
        new_values_en_2 = self.make_new_values(self.opening_values_raw['en'], 2)
        content_en_2 = self.plugin_values_raw['en'].format(2)
        self.create_revision(
            job_opening, content=content_en_2, **new_values_en_2)

        # revert to 1
        job_opening = JobOpening.objects.get(pk=job_opening.pk)
        self.revert_to(job_opening, 1)
        job_opening = JobOpening.objects.get(pk=job_opening.pk)
        with switch_language(job_opening, 'en'):
            url = job_opening.get_absolute_url()
        response = self.client.get(url)
        self.assertContains(response, new_values_en_1['title'])
        self.assertContains(response, content_en_1)
        self.assertNotContains(response, new_values_en_2['title'])
        self.assertNotContains(response, content_en_2)

    def test_edit_plugin_directly(self):
        job_opening = self.create_default_job_opening()

        # revision 1
        content_en_1 = self.plugin_values_raw['en'].format(1)
        self.create_revision(job_opening, content=content_en_1)

        self.assertEqual(len(get_for_object(job_opening)), 1)

        # revision 2
        content_en_2 = self.plugin_values_raw['en'].format(2)
        with transaction.atomic():
            language = job_opening.get_current_language()
            plugins = job_opening.content.get_plugins().filter(
                language=language)
            plugin = plugins[0].get_plugin_instance()[0]
            plugin.body = content_en_2
            plugin.save()
            aldryn_create_revision(job_opening)
        self.assertEqual(len(get_for_object(job_opening)), 2)

        with switch_language(job_opening, 'en'):
            url = job_opening.get_absolute_url()
        response = self.client.get(url)

        self.assertContains(response, content_en_2)
        self.assertNotContains(response, content_en_1)

        self.revert_to(job_opening, 1)
        job_opening = JobOpening.objects.get(pk=job_opening.pk)
        response = self.client.get(job_opening.get_absolute_url())
        self.assertContains(response, content_en_1)
        self.assertNotContains(response, content_en_2)

    def test_category_revert_revision_correct_for_diverged_translations(self):
        category = self.default_category
        self.assertEqual(len(get_for_object(category)), 0)
        # revision 1: en 1, de 0
        new_values = self.make_new_values(self.category_values_raw['en'], 1)
        with switch_language(category, 'en'):
            self.create_revision(category, **new_values)

        # revision 2: en 1, de 1
        new_values_2 = self.make_new_values(self.category_values_raw['de'], 1)
        with switch_language(category, 'de'):
            self.create_revision(category, **new_values_2)

        # revision 3: en 1, de 2
        new_values_3 = self.make_new_values(self.category_values_raw['de'], 2)
        with switch_language(category, 'de'):
            self.create_revision(category, **new_values_3)

        # revision 4: en 2, de 2
        with switch_language(category, 'en'):
            new_values_4 = self.make_new_values(
                self.category_values_raw['en'], 2)
            self.create_revision(category, **new_values_4)

        # revert to 3: en 1, de 2
        self.revert_to(category, 3)
        category = JobCategory.objects.get(pk=category.pk)
        # test en values
        # switch_language(category,
        with switch_language(category, 'en'):
            for prop in new_values.keys():
                self.assertEqual(getattr(category, prop), new_values[prop])
        # test de values
        with switch_language(category, 'de'):
            for prop in new_values_3.keys():
                self.assertEqual(getattr(category, prop), new_values_3[prop])

    def test_opening_revert_revision_correct_for_diverged_translations(self):
        opening = self.create_default_job_opening(translated=True)
        self.assertEqual(len(get_for_object(opening)), 0)
        initial_values = {
            'category': self.default_category,
            'is_active': True,
            'can_apply': True,
        }
        # revision 1: en 1, de 0
        new_values_en_1 = self.make_new_values(self.opening_values_raw['en'], 1)
        with switch_language(opening, 'en'):
            self.create_revision(opening, **new_values_en_1)

        # revision 2: en 1, de 1
        new_values_de_1 = self.make_new_values(self.opening_values_raw['de'], 1)
        with switch_language(opening, 'de'):
            self.create_revision(opening, **new_values_de_1)

        # revision 3: en 1, de 2
        new_values_de_2 = self.make_new_values(self.opening_values_raw['de'], 2)
        with switch_language(opening, 'de'):
            self.create_revision(opening, **new_values_de_2)

        # revision 4: en 2, de 2, also contains updated category and bool fields
        new_values_en_2 = self.make_new_values(self.opening_values_raw['en'], 2)
        new_category = JobCategory.objects.create(
            app_config=self.app_config,
            **self.make_new_values(self.category_values_raw['en'], 2))
        new_values_en_2['category'] = new_category
        new_values_en_2['is_active'] = False
        new_values_en_2['can_apply'] = False
        with switch_language(opening, 'en'):
            self.create_revision(opening, **new_values_en_2)

        # revert to 3: en 1, de 2
        self.revert_to(opening, 3)
        opening = JobOpening.objects.get(pk=opening.pk)
        # test en values
        with switch_language(opening, 'en'):
            for prop in new_values_en_1.keys():
                self.assertEqual(getattr(opening, prop), new_values_en_1[prop])
            # test untranslated fields, should be the same as atm of reverted
            # revision
            for prop in initial_values.keys():
                self.assertEqual(getattr(opening, prop), initial_values[prop])

        # test de values
        with switch_language(opening, 'de'):
            for prop in new_values_de_2.keys():
                self.assertEqual(getattr(opening, prop), new_values_de_2[prop])
            # test untranslated fields shouldn't be changed regardless of
            # language
            for prop in initial_values.keys():
                self.assertEqual(getattr(opening, prop), initial_values[prop])

    def test_opening_revert_revision_serves_for_diverged_translations(self):
        opening = self.create_default_job_opening(translated=True)
        self.assertEqual(len(get_for_object(opening)), 0)

        # revision 1: en 1, de 0
        new_values_en_1 = self.make_new_values(self.opening_values_raw['en'], 1)
        content_en_1 = self.plugin_values_raw['en'].format(1)
        with switch_language(opening, 'en'):
            self.create_revision(
                opening, content=content_en_1, **new_values_en_1)

        # revision 2: en 1, de 1
        new_values_de_1 = self.make_new_values(self.opening_values_raw['de'], 1)
        content_de_1 = self.plugin_values_raw['de'].format(1)
        with switch_language(opening, 'de'):
            self.create_revision(
                opening, content=content_de_1, **new_values_de_1)

        # revision 3: en 1, de 2
        new_values_de_2 = self.make_new_values(self.opening_values_raw['de'], 2)
        content_de_2 = self.plugin_values_raw['de'].format(2)
        with switch_language(opening, 'de'):
            self.create_revision(
                opening, content=content_de_2, **new_values_de_2)

        # revision 4: en 2, de 2
        content_en_2 = self.plugin_values_raw['en'].format(2)
        new_values_en_2 = self.make_new_values(self.opening_values_raw['en'], 2)
        with switch_language(opening, 'en'):
            self.create_revision(
                opening, content=content_en_2, **new_values_en_2)

        # revert to 3: en 1, de 2
        self.revert_to(opening, 3)
        opening = JobOpening.objects.get(pk=opening.pk)
        # test en values
        with switch_language(opening, 'en'):
            url_reverted_en = opening.get_absolute_url()
        response = self.client.get(url_reverted_en)
        self.assertContains(response, new_values_en_1['title'])
        self.assertContains(response, content_en_1)
        # ensure that there is no latest values (rev 4)
        self.assertNotContains(response, new_values_en_2['title'])
        self.assertNotContains(response, content_en_2)
        # test de values
        with switch_language(opening, 'de'):
            url_reverted_de = opening.get_absolute_url()
        response = self.client.get(url_reverted_de)
        self.assertContains(response, new_values_de_2['title'])
        self.assertContains(response, content_de_2)
        # ensure that there is no previous values (rev 2)
        self.assertNotContains(response, new_values_de_1['title'])
        self.assertNotContains(response, content_de_1)

    def test_opening_is_reverted_if_fk_object_was_deleted(self):
        opening = self.create_default_job_opening(translated=True)
        self.assertEqual(len(get_for_object(opening)), 0)

        # revision 1: en 1, de 0
        category_1 = self.default_category
        new_values_en_1 = self.make_new_values(self.opening_values_raw['en'], 1)
        with switch_language(opening, 'en'):
            self.create_revision(opening, **new_values_en_1)

        # revision 2: en 1, de 1
        new_values_de_1 = self.make_new_values(self.opening_values_raw['de'], 1)
        # update category
        category_2 = JobCategory.objects.create(
            app_config=self.app_config,
            **self.make_new_values(self.category_values_raw['en'], 2))
        new_values_de_1['category'] = category_2
        with switch_language(opening, 'de'):
            self.create_revision(opening, **new_values_de_1)

        # delete category_1
        with transaction.atomic():
            category_1.delete()
        self.assertEqual(JobCategory.objects.all().count(), 1)

        # revert to 1
        self.revert_to(opening, 1)
        opening = JobOpening.objects.get(pk=opening.pk)
        self.assertNotEqual(opening.category, category_2)
        self.assertEqual(JobCategory.objects.all().count(), 2)

    # JobApplication
    def test_job_application_revision_is_created(self):
        job_opening = self.create_default_job_opening()
        application = JobApplication.objects.create(
            job_opening=job_opening,
            **self.application_default_values)
        self.assertEqual(len(get_for_object(application)), 0)

        # revision 1
        new_values_1 = self.make_new_values(self.application_values_raw, 1)
        self.create_revision(application, **new_values_1)
        self.assertEqual(len(get_for_object(application)), 1)

    def test_job_application_contains_latest_values(self):
        job_opening = self.create_default_job_opening()
        application = JobApplication.objects.create(
            job_opening=job_opening,
            **self.application_default_values)

        # revision 1
        new_values_1 = self.make_new_values(self.application_values_raw, 1)
        self.create_revision(application, **new_values_1)
        self.assertEqual(len(get_for_object(application)), 1)

        application = JobApplication.objects.get(pk=application.pk)
        for prop in new_values_1.keys():
            self.assertEqual(getattr(application, prop), new_values_1[prop])

        # revision 2
        new_values_2 = self.make_new_values(self.application_values_raw, 2)
        new_opening = JobOpening.objects.create(
            category=self.default_category,
            **self.make_new_values(self.opening_values_raw['en'], 2))
        new_values_2['job_opening'] = new_opening
        self.create_revision(application, **new_values_2)
        self.assertEqual(len(get_for_object(application)), 2)

        application = JobApplication.objects.get(pk=application.pk)
        for prop in new_values_2.keys():
            self.assertEqual(getattr(application, prop), new_values_2[prop])

    def test_job_application_revision_is_reverted(self):
        job_opening = self.create_default_job_opening()
        application = JobApplication.objects.create(
            job_opening=job_opening, **self.application_default_values)

        # revision 1
        new_values_1 = self.make_new_values(self.application_values_raw, 1)
        self.create_revision(application, **new_values_1)
        self.assertEqual(len(get_for_object(application)), 1)

        # revision 2
        new_values_2 = self.make_new_values(self.application_values_raw, 2)
        new_opening = JobOpening.objects.create(
            category=self.default_category,
            **self.make_new_values(self.opening_values_raw['en'], 2))
        new_values_2['job_opening'] = new_opening
        self.create_revision(application, **new_values_2)
        self.assertEqual(len(get_for_object(application)), 2)

        # revert to 1
        self.revert_to(application, 1)
        application = JobApplication.objects.get(pk=application.pk)
        # add initial job_opening to values list to be checked
        new_values_1['job_opening'] = job_opening
        for prop in new_values_1.keys():
            self.assertEqual(getattr(application, prop), new_values_1[prop])

    def test_job_application_revision_is_reverted_if_fk_obj_was_deleted(self):
        job_opening = self.create_default_job_opening()
        application = JobApplication.objects.create(
            job_opening=job_opening, **self.application_default_values)

        # revision 1
        new_values_1 = self.make_new_values(self.application_values_raw, 1)
        self.create_revision(application, **new_values_1)
        self.assertEqual(len(get_for_object(application)), 1)

        # revision 2
        new_values_2 = self.make_new_values(self.application_values_raw, 2)
        new_opening = JobOpening.objects.create(
            category=self.default_category,
            **self.make_new_values(self.opening_values_raw['en'], 2))
        new_values_2['job_opening'] = new_opening
        self.create_revision(application, **new_values_2)
        self.assertEqual(len(get_for_object(application)), 2)

        # delete initial opening
        with transaction.atomic():
            job_opening.delete()

        # revert to 1
        self.revert_to(application, 1)
        application = JobApplication.objects.get(pk=application.pk)
        self.assertEqual(JobOpening.objects.all().count(), 2)
        # get restored job opening
        job_opening = JobOpening.objects.all().exclude(pk=new_opening.pk).get()
        # add initial job_opening to values list to be checked
        new_values_1['job_opening'] = job_opening
        for prop in new_values_1.keys():
            self.assertEqual(getattr(application, prop), new_values_1[prop])



# -*- coding: utf-8 -*-
import re

from django.conf import settings

from south.v2 import DataMigration

from ..models import default_jobs_attachment_upload_to

class Migration(DataMigration):

    def forwards(self, orm):
        "Write your forwards methods here."
        if not getattr(settings, 'ALDRYN_JOBS_ATTACHMENT_UPLOAD_DIR', None):
            attachment_fields = ['attachment', 'attachment_2', 'attachment_3', 'attachment_4']
            has_random_component = re.compile(r'(?P<token>[0-9a-z]{8}-[0-9a-z]{4}-[0-9a-z]{4}-[0-9a-z]{4}-[0-9a-z]{12})')
            for job_application in orm['aldryn_jobs.jobapplication'].objects.all():
                for field in attachment_fields:
                    attachment = getattr(job_application, field, None)
                    if attachment:
                        storage = attachment.storage
                        oldpath = attachment.path
                        if not has_random_component.search(attachment.url) and storage.exists(oldpath):
                            name = default_jobs_attachment_upload_to(job_application, attachment.name)
                            path = attachment.storage.save(name, attachment.file)
                            setattr(job_application, field, path)
                            job_application.save()
                            storage.delete(oldpath)

    def backwards(self, orm):
        "Write your backwards methods here."

    models = {
        u'aldryn_jobs.jobapplication': {
            'Meta': {'ordering': "['-created']", 'object_name': 'JobApplication'},
            'attachment': ('django.db.models.fields.files.FileField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'attachment_2': ('django.db.models.fields.files.FileField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'attachment_3': ('django.db.models.fields.files.FileField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'attachment_4': ('django.db.models.fields.files.FileField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'cover_letter': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '20'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'job_offer': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['aldryn_jobs.JobOffer']"}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '20'})
        },
        u'aldryn_jobs.jobcategory': {
            'Meta': {'ordering': "['ordering']", 'object_name': 'JobCategory'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ordering': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'supervisors': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'job_offer_categories'", 'blank': 'True', 'to': u"orm['auth.User']"})
        },
        u'aldryn_jobs.jobcategorytranslation': {
            'Meta': {'unique_together': "[['slug', 'language_code'], ('language_code', 'master')]", 'object_name': 'JobCategoryTranslation', 'db_table': "u'aldryn_jobs_jobcategory_translation'"},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'null': 'True', 'to': u"orm['aldryn_jobs.JobCategory']"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '255', 'blank': 'True'})
        },
        u'aldryn_jobs.joboffer': {
            'Meta': {'ordering': "['category__ordering', 'category', '-created']", 'object_name': 'JobOffer'},
            'can_apply': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'category': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'jobs'", 'to': u"orm['aldryn_jobs.JobCategory']"}),
            'content': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cms.Placeholder']", 'null': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'publication_end': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'publication_start': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'})
        },
        u'aldryn_jobs.joboffertranslation': {
            'Meta': {'unique_together': "[['slug', 'language_code'], ('language_code', 'master')]", 'object_name': 'JobOfferTranslation', 'db_table': "u'aldryn_jobs_joboffer_translation'"},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'null': 'True', 'to': u"orm['aldryn_jobs.JobOffer']"}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '255', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'cms.placeholder': {
            'Meta': {'object_name': 'Placeholder'},
            'default_width': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'slot': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        }
    }

    complete_apps = ['aldryn_jobs']
    symmetrical = True



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from .forms import (
    JobListPluginForm,
    JobCategoriesListPluginForm,
)
from .models import (
    JobListPlugin,
    JobCategoriesPlugin,
    JobOpening,
)
from .utils import namespace_is_apphooked


class NameSpaceCheckMixin(object):

    def render(self, context, instance, placeholder):
        # check if we have a valid app_config that is app hooked to a page.
        # so that we won't have a 500 error if page with that app hook
        # was deleted.
        namespace = (instance.app_config.namespace if instance.app_config
                     else '')

        if not namespace_is_apphooked(namespace):
            context['plugin_configuration_error'] = _(
                'There is an error in plugin configuration: selected Job '
                'config is not available. Please switch to edit mode and '
                'change plugin app_config settings to use valid config. '
                'Also note that aldryn-jobs should be used at least once '
                'as an apphook for that config.')

        return super(NameSpaceCheckMixin, self).render(
            context, instance, placeholder)


class JobCategoriesList(NameSpaceCheckMixin, CMSPluginBase):
    model = JobCategoriesPlugin
    form = JobCategoriesListPluginForm
    module = 'Jobs'
    name = _('Categories list')
    render_template = 'aldryn_jobs/plugins/categories_list.html'


class JobList(NameSpaceCheckMixin, CMSPluginBase):
    form = JobListPluginForm
    model = JobListPlugin
    module = "Jobs"
    name = _('Job List')
    render_template = 'aldryn_jobs/plugins/latest_entries.html'

    def render(self, context, instance, placeholder):
        context = super(JobList, self).render(context, instance, placeholder)
        namespace = (instance.app_config.namespace if instance.app_config
                     else '')
        if namespace == '' or context.get('plugin_configuration_error', False):
            vacancies = JobOpening.objects.none()
        else:
            vacancies = instance.job_openings(namespace)
        context['vacancies'] = vacancies
        context['vacancies_count'] = len(vacancies)
        return context

plugin_pool.register_plugin(JobCategoriesList)
plugin_pool.register_plugin(JobList)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.db import transaction
from django.contrib import messages
from django.http import Http404
from django.shortcuts import redirect
from django.utils.translation import (
    ugettext as _, get_language_from_request
)
from django.views.generic import DetailView, ListView
from aldryn_apphooks_config.mixins import AppConfigMixin
from aldryn_apphooks_config.utils import get_app_instance
from menus.utils import set_language_changer
from parler.views import TranslatableSlugMixin
from reversion.revisions import revision_context_manager

from .forms import JobApplicationForm
from .models import JobCategory, JobOpening


class JobsBaseMixin(object):
    template_name = 'aldryn_jobs/jobs_list.html'
    model = JobOpening

    def dispatch(self, request, *args, **kwargs):
        # prepare language for misc usage
        self.language = get_language_from_request(request, check_path=True)
        return super(JobsBaseMixin, self).dispatch(request, *args, **kwargs)

    def get_queryset(self):
        """
        Base queryset returns active JobOpenings with respect to language and
        namespace. selects related categories, no ordering.
        """
        # if config is none - probably apphook relaod is in progress, or
        # something is wrong, anyway do not fail with 500
        if self.config is None:
            return JobOpening.objects.none()
        return (
            JobOpening.objects.active()
                              .namespace(self.config.namespace)
                              .language(self.language)
                              .active_translations(self.language)
                              .select_related('category')
        )


class JobOpeningList(JobsBaseMixin, AppConfigMixin, ListView):

    def get_queryset(self):
        return super(JobOpeningList, self).get_queryset().order_by(
            'category__ordering', 'ordering')


class CategoryJobOpeningList(JobsBaseMixin, AppConfigMixin, ListView):
    def get_queryset(self):
        category_slug = self.kwargs['category_slug']
        try:
            self.category = (
                JobCategory.objects
                           .language(self.language)
                           .active_translations(self.language,
                                                slug=category_slug)
                           .namespace(self.namespace)
                           .get()
            )
        except JobCategory.DoesNotExist:
            raise Http404

        self.set_language_changer(category=self.category)
        return (super(CategoryJobOpeningList, self).get_queryset()
                .filter(category=self.category)
                .order_by('ordering'))

    def set_language_changer(self, category):
        """Translate the slug while changing the language."""
        set_language_changer(self.request, category.get_absolute_url)


class JobOpeningDetail(AppConfigMixin, TranslatableSlugMixin, DetailView):
    model = JobOpening
    form_class = JobApplicationForm
    template_name = 'aldryn_jobs/jobs_detail.html'
    slug_url_kwarg = 'job_opening_slug'

    def dispatch(self, request, *args, **kwargs):
        self.request = request
        self.namespace, self.config = get_app_instance(request)
        self.object = self.get_object()
        self.set_language_changer(self.object)
        return super(JobOpeningDetail, self).dispatch(request, *args, **kwargs)

    def get_form_class(self):
        return self.form_class

    def get_form_kwargs(self):
        """
        Returns the keyword arguments for instantiating the form.
        """
        kwargs = {'job_opening': self.object}

        if self.request.method in ('POST', 'PUT'):
            kwargs.update({
                'data': self.request.POST,
                'files': self.request.FILES,
            })
        return kwargs

    def get_form(self, form_class):
        """
        Returns an instance of the form to be used in this view.
        """
        return form_class(**self.get_form_kwargs())

    def set_language_changer(self, job_opening):
        """Translate the slug while changing the language."""
        set_language_changer(self.request, job_opening.get_absolute_url)

    def get(self, *args, **kwargs):
        form_class = self.get_form_class()
        self.form = self.get_form(form_class)
        return super(JobOpeningDetail, self).get(*args, **kwargs)

    def get_queryset(self):
        qs = super(JobOpeningDetail, self).get_queryset()
        return qs.namespace(self.namespace)

    @transaction.atomic
    @revision_context_manager.create_revision()
    def post(self, *args, **kwargs):
        """Handles application for the job."""
        if not self.object.can_apply:
            messages.success(self.request,
                _("You can't apply for this job."))
            return redirect(self.object.get_absolute_url())

        form_class = self.get_form_class()
        self.form = self.get_form(form_class)

        if self.form.is_valid():
            self.form.save()
            msg = _("You have successfully applied for %(job)s.") % {
                'job': self.object.title
            }
            messages.success(self.request, msg)
            return redirect(self.object.get_absolute_url())
        else:
            return super(JobOpeningDetail, self).get(*args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(JobOpeningDetail, self).get_context_data(**kwargs)
        context['form'] = self.form
        return context



from django.utils.translation import override
from cms import api

from ..models import JobCategory, JobsConfig

from .base import JobsBaseTestCase


class TestAppConfigPluginsMixin(object):
    plugin_to_test = 'TextPlugin'
    plugin_params = {}

    def setUp(self):
        super(TestAppConfigPluginsMixin, self).setUp()
        self.plugin_page = self.create_plugin_page()
        self.placeholder = self.plugin_page.placeholders.all()[0]

    def create_plugin_page(self):
        page = api.create_page(
            title="Jobs plugin page en",
            template=self.template,
            language=self.language,
            parent=self.root_page,
            published=True)
        api.create_title('de', 'Jobs plugin page de', page,
                         slug='jobs-plugin-page-de')
        page.publish('en')
        page.publish('de')
        return page.reload()

    def _create_plugin(self, page, language, app_config, **plugin_params):
        """
        Create plugin of type self.plugin_to_test and plugin_params in
        given language to a page placeholder.
        Assumes that page has that translation.
        """
        placeholder = page.placeholders.all()[0]
        plugin = api.add_plugin(
            placeholder, self.plugin_to_test, language,
            app_config=app_config, **plugin_params)
        page.publish(language)
        return plugin


class TestPluginFailuresWithDeletedAppHookMixin(object):
    """
    General test cases for 500 errors if app config or page wtih app hook
    was deleted.
    Relies on setUp which should prepare all plugins
    """

    def test_plugin_doesnt_break_page_if_configured_apphook_was_deleted(self):
        # delete apphooked page
        self.page.delete()

        for language_code in ('en', 'de'):
            with override(language_code):
                page_url_en = self.plugin_page.get_absolute_url()
            response = self.client.get(page_url_en)
            self.assertEqual(response.status_code, 200)

    def test_plugin_doesnt_break_page_for_su_if_apphook_was_deleted(self):
        # delete apphooked page
        self.page.delete()

        for language_code in ('en', 'de'):
            with override(language_code):
                page_url_en = '{0}?edit'.format(
                    self.plugin_page.get_absolute_url())

            login_result = self.client.login(
                username=self.super_user, password=self.super_user_password)
            self.assertEqual(login_result, True)

            response = self.client.get(page_url_en)
            self.assertEqual(response.status_code, 200)

    def test_plugin_does_not_breaks_page_if_job_config_was_deleted(self):
        # delete JobConfig
        self.app_config.delete()

        for language_code in ('en', 'de'):
            with override(language_code):
                page_url_en = self.plugin_page.get_absolute_url()
            response = self.client.get(page_url_en)
            self.assertEqual(response.status_code, 200)

    def test_plugin_doesnt_breaks_page_for_su_if_job_config_was_deleted(self):
        # delete JobConfig
        self.app_config.delete()

        for language_code in ('en', 'de'):
            with override(language_code):
                page_url_en = '{0}?edit'.format(
                    self.plugin_page.get_absolute_url())

            login_result = self.client.login(
                username=self.super_user, password=self.super_user_password)
            self.assertEqual(login_result, True)

            response = self.client.get(page_url_en)
            self.assertEqual(response.status_code, 200)

    def test_plugin_does_not_displays_error_message_to_non_super_users(self):
        # delete apphooked page
        self.page.delete()

        with override('en'):
            page_url = self.plugin_page.get_absolute_url()

        response = self.client.get(page_url)
        self.assertEqual(response.status_code, 200)
        self.assertNotContains(
            response,
            'There is an error in plugin configuration: selected Job')

    def test_plugin_displays_error_message_to_super_users(self):
        # delete apphooked page
        self.page.delete()

        with override('en'):
            page_url = self.plugin_page.get_absolute_url()

        login_result = self.client.login(
            username=self.super_user, password=self.super_user_password)
        self.assertEqual(login_result, True)

        response = self.client.get(page_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response,
            'There is an error in plugin configuration: selected Job')


class TestJobCategoriesListPlugin(TestAppConfigPluginsMixin,
                                  TestPluginFailuresWithDeletedAppHookMixin,
                                  JobsBaseTestCase):
    plugin_to_test = 'JobCategoriesList'

    def setUp(self):
        super(TestJobCategoriesListPlugin, self).setUp()
        # create plugin for both languages
        self.create_plugin(self.plugin_page, 'en', self.app_config)
        self.create_plugin(self.plugin_page, 'de', self.app_config)

    def create_plugin(self, page, language, app_config,
                      **plugin_params):
        plugin = self._create_plugin(
            page, language, app_config, **plugin_params)
        return plugin


class TestJobListPlugin(TestAppConfigPluginsMixin,
                        TestPluginFailuresWithDeletedAppHookMixin,
                        JobsBaseTestCase):
    plugin_to_test = 'JobList'

    def setUp(self):
        super(TestJobListPlugin, self).setUp()
        self.job_opening = self.create_default_job_opening(translated=True)
        self.plugin_en = self.create_plugin(self.plugin_page, 'en',
            self.app_config, jobopenings=self.job_opening)
        self.plugin_de = self.create_plugin(self.plugin_page, 'de',
            self.app_config, jobopenings=self.job_opening)

    def create_plugin(self, page, language, app_config, jobopenings=None,
                      **plugin_params):
        plugin = self._create_plugin(
            page, language, app_config, **plugin_params)
        if jobopenings is not None:
            # we need to update plugin configuration model with correct group
            # it is located under it's own manager
            plugin.joblistplugin.jobopenings.add(
                jobopenings)
            plugin.save()
        return plugin

    def test_list_plugin_shows_selected_items(self):
        # since we setting up plugin in a setUp method with default
        # job opening and default config, at this point it should
        # contain all that we need.
        with override('en'):
            page_url = self.plugin_page.get_absolute_url()
            default_opening_url = self.job_opening.get_absolute_url()
        response = self.client.get(page_url)
        self.assertContains(response, self.job_opening.title)
        self.assertContains(response, default_opening_url)

    def test_list_plugin_doesnt_shows_delayed_openings(self):
        new_opening = self.create_new_job_opening(
            self.prepare_data(1, update_date=True))
        self.plugin_en.joblistplugin.jobopenings.add(new_opening)
        self.plugin_en.save()
        self.plugin_page.publish('en')
        with override('en'):
            page_url = self.plugin_page.get_absolute_url()
            opening_url = new_opening.get_absolute_url()
            default_opening_url = self.job_opening.get_absolute_url()
        response = self.client.get(page_url)
        self.assertNotContains(response, new_opening.title)
        self.assertNotContains(response, opening_url)

        # test that default job opening is still present
        self.assertContains(response, self.job_opening.title)
        self.assertContains(response, default_opening_url)

    def test_list_plugin_doesnt_shows_job_openings_from_other_config(self):
        new_config = JobsConfig.objects.create(namespace='different_namespace')
        # prepare apphook
        self.create_page(
            title='new apphook', slug='new-apphook',
            namespace=new_config.namespace)
        new_category = JobCategory.objects.create(
            name='Completely different namespace',
            slug='completely-different-namespace',
            app_config=new_config)
        new_opening = self.create_new_job_opening(
            self.prepare_data(1, category=new_category))
        self.plugin_en.joblistplugin.jobopenings.add(new_opening)
        self.plugin_en.app_config = new_config
        self.plugin_en.save()
        self.plugin_page.publish('en')
        with override('en'):
            page_url = self.plugin_page.get_absolute_url()
            default_opening_url = self.job_opening.get_absolute_url()
            new_opening_url = new_opening.get_absolute_url()
        response = self.client.get(page_url)
        # check that app config is repsected by plugin
        self.assertContains(response, new_opening.title)
        self.assertContains(response, new_opening_url)
        # check that there is no openings from other config
        self.assertNotContains(response, self.job_opening.title)
        self.assertNotContains(response, default_opening_url)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'JobCategoryTranslation'
        db.create_table('aldryn_jobs_jobcategory_translation', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('slug', self.gf('django.db.models.fields.SlugField')(max_length=255)),
            ('language_code', self.gf('django.db.models.fields.CharField')(max_length=15, db_index=True)),
            ('master', self.gf('django.db.models.fields.related.ForeignKey')(related_name='translations', null=True, to=orm['aldryn_jobs.JobCategory'])),
        ))
        db.send_create_signal('aldryn_jobs', ['JobCategoryTranslation'])

        # Adding unique constraint on 'JobCategoryTranslation', fields ['language_code', 'master']
        db.create_unique('aldryn_jobs_jobcategory_translation', ['language_code', 'master_id'])

        # Adding model 'JobCategory'
        db.create_table('aldryn_jobs_jobcategory', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('ordering', self.gf('django.db.models.fields.IntegerField')()),
        ))
        db.send_create_signal('aldryn_jobs', ['JobCategory'])

        # Adding model 'JobOfferTranslation'
        db.create_table('aldryn_jobs_joboffer_translation', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('slug', self.gf('django.db.models.fields.SlugField')(max_length=255)),
            ('language_code', self.gf('django.db.models.fields.CharField')(max_length=15, db_index=True)),
            ('master', self.gf('django.db.models.fields.related.ForeignKey')(related_name='translations', null=True, to=orm['aldryn_jobs.JobOffer'])),
        ))
        db.send_create_signal('aldryn_jobs', ['JobOfferTranslation'])

        # Adding unique constraint on 'JobOfferTranslation', fields ['language_code', 'master']
        db.create_unique('aldryn_jobs_joboffer_translation', ['language_code', 'master_id'])

        # Adding model 'JobOffer'
        db.create_table('aldryn_jobs_joboffer', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('content', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['cms.Placeholder'], null=True)),
            ('category', self.gf('django.db.models.fields.related.ForeignKey')(related_name='jobs', to=orm['aldryn_jobs.JobCategory'])),
            ('active', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
        ))
        db.send_create_signal('aldryn_jobs', ['JobOffer'])


    def backwards(self, orm):
        # Removing unique constraint on 'JobOfferTranslation', fields ['language_code', 'master']
        db.delete_unique('aldryn_jobs_joboffer_translation', ['language_code', 'master_id'])

        # Removing unique constraint on 'JobCategoryTranslation', fields ['language_code', 'master']
        db.delete_unique('aldryn_jobs_jobcategory_translation', ['language_code', 'master_id'])

        # Deleting model 'JobCategoryTranslation'
        db.delete_table('aldryn_jobs_jobcategory_translation')

        # Deleting model 'JobCategory'
        db.delete_table('aldryn_jobs_jobcategory')

        # Deleting model 'JobOfferTranslation'
        db.delete_table('aldryn_jobs_joboffer_translation')

        # Deleting model 'JobOffer'
        db.delete_table('aldryn_jobs_joboffer')


    models = {
        'aldryn_jobs.jobcategory': {
            'Meta': {'ordering': "['ordering']", 'object_name': 'JobCategory'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ordering': ('django.db.models.fields.IntegerField', [], {})
        },
        'aldryn_jobs.jobcategorytranslation': {
            'Meta': {'unique_together': "[('language_code', 'master')]", 'object_name': 'JobCategoryTranslation', 'db_table': "'aldryn_jobs_jobcategory_translation'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'null': 'True', 'to': "orm['aldryn_jobs.JobCategory']"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '255'})
        },
        'aldryn_jobs.joboffer': {
            'Meta': {'ordering': "['-created']", 'object_name': 'JobOffer'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'category': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'jobs'", 'to': "orm['aldryn_jobs.JobCategory']"}),
            'content': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cms.Placeholder']", 'null': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        'aldryn_jobs.joboffertranslation': {
            'Meta': {'unique_together': "[('language_code', 'master')]", 'object_name': 'JobOfferTranslation', 'db_table': "'aldryn_jobs_joboffer_translation'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'null': 'True', 'to': "orm['aldryn_jobs.JobOffer']"}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '255'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'cms.placeholder': {
            'Meta': {'object_name': 'Placeholder'},
            'default_width': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'slot': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'})
        }
    }

    complete_apps = ['aldryn_jobs']


# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from os.path import splitext
from django.core.urlresolvers import reverse, NoReverseMatch

from django.utils.text import get_valid_filename as get_valid_filename_django
from django.template.defaultfilters import slugify


def get_valid_filename(s):
    """
    like the regular get_valid_filename, but also slugifies away umlauts and
    stuff. Copied from django-filer
    """
    s = get_valid_filename_django(s)
    filename, ext = splitext(s)
    filename = slugify(filename)
    ext = slugify(ext)
    if ext:
        return "%s.%s" % (filename, ext)
    else:
        return "%s" % (filename,)


def namespace_is_apphooked(namespace):
    # avoid circular import
    from .urls import DEFAULT_VIEW
    """
    Check if provided namespace has an app-hooked page.
    Returns True or False.
    """
    try:
        reverse('{0}:{1}'.format(namespace, DEFAULT_VIEW))
    except NoReverseMatch:
        return False
    return True



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'JobApplication'
        db.create_table('aldryn_jobs_jobapplication', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('job_offer', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['aldryn_jobs.JobOffer'])),
            ('first_name', self.gf('django.db.models.fields.CharField')(max_length=20)),
            ('last_name', self.gf('django.db.models.fields.CharField')(max_length=20)),
            ('email', self.gf('django.db.models.fields.EmailField')(max_length=75)),
            ('cover_letter', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('attachment', self.gf('django.db.models.fields.files.FileField')(max_length=100, null=True, blank=True)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
        ))
        db.send_create_signal('aldryn_jobs', ['JobApplication'])

        # Adding field 'JobOffer.can_apply'
        db.add_column('aldryn_jobs_joboffer', 'can_apply',
                      self.gf('django.db.models.fields.BooleanField')(default=True),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting model 'JobApplication'
        db.delete_table('aldryn_jobs_jobapplication')

        # Deleting field 'JobOffer.can_apply'
        db.delete_column('aldryn_jobs_joboffer', 'can_apply')


    models = {
        'aldryn_jobs.jobapplication': {
            'Meta': {'object_name': 'JobApplication'},
            'attachment': ('django.db.models.fields.files.FileField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'cover_letter': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '20'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'job_offer': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['aldryn_jobs.JobOffer']"}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '20'})
        },
        'aldryn_jobs.jobcategory': {
            'Meta': {'ordering': "['ordering']", 'object_name': 'JobCategory'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ordering': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        },
        'aldryn_jobs.jobcategorytranslation': {
            'Meta': {'unique_together': "[['slug', 'language_code'], ('language_code', 'master')]", 'object_name': 'JobCategoryTranslation', 'db_table': "'aldryn_jobs_jobcategory_translation'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'null': 'True', 'to': "orm['aldryn_jobs.JobCategory']"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '255', 'blank': 'True'})
        },
        'aldryn_jobs.joboffer': {
            'Meta': {'ordering': "['category__ordering', 'category', '-created']", 'object_name': 'JobOffer'},
            'can_apply': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'category': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'jobs'", 'to': "orm['aldryn_jobs.JobCategory']"}),
            'content': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cms.Placeholder']", 'null': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'publication_end': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'publication_start': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'})
        },
        'aldryn_jobs.joboffertranslation': {
            'Meta': {'unique_together': "[['slug', 'language_code'], ('language_code', 'master')]", 'object_name': 'JobOfferTranslation', 'db_table': "'aldryn_jobs_joboffer_translation'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'null': 'True', 'to': "orm['aldryn_jobs.JobOffer']"}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '255', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'cms.placeholder': {
            'Meta': {'object_name': 'Placeholder'},
            'default_width': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'slot': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'})
        }
    }

    complete_apps = ['aldryn_jobs']


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import app_data.fields
import djangocms_text_ckeditor.fields
from django.conf import settings
import cms.models.fields
import aldryn_jobs.models
import sortedm2m.fields


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('cms', '0003_auto_20140926_2347'),
    ]

    operations = [
        migrations.CreateModel(
            name='JobApplication',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True, serialize=False)),
                ('salutation', models.CharField(verbose_name='salutation', max_length=20, choices=[('male', 'Mr.'), ('female', 'Mrs.')], blank=True, default='male')),
                ('first_name', models.CharField(verbose_name='first name', max_length=20)),
                ('last_name', models.CharField(verbose_name='last name', max_length=20)),
                ('email', models.EmailField(verbose_name='email', max_length=75)),
                ('cover_letter', models.TextField(verbose_name='cover letter', blank=True)),
                ('created', models.DateTimeField(verbose_name='created', auto_now_add=True)),
                ('is_rejected', models.BooleanField(verbose_name='rejected?', default=False)),
                ('rejection_date', models.DateTimeField(verbose_name='rejection date', blank=True, null=True)),
            ],
            options={
                'verbose_name': 'job application',
                'verbose_name_plural': 'job applications',
                'ordering': ['-created'],
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='JobApplicationAttachment',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True, serialize=False)),
                ('file', models.FileField(null=True, max_length=200, upload_to=aldryn_jobs.models.default_jobs_attachment_upload_to, blank=True)),
                ('application', models.ForeignKey(verbose_name='job application', related_name='attachments', to='aldryn_jobs.JobApplication')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='JobCategoriesPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, parent_link=True, primary_key=True, serialize=False, to='cms.CMSPlugin')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='JobCategory',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True, serialize=False)),
                ('ordering', models.IntegerField(verbose_name='ordering', default=0)),
            ],
            options={
                'verbose_name': 'job category',
                'verbose_name_plural': 'job categories',
                'ordering': ['ordering'],
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='JobCategoryTranslation',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True, serialize=False)),
                ('language_code', models.CharField(db_index=True, verbose_name='Language', max_length=15)),
                ('name', models.CharField(verbose_name='name', max_length=255)),
                ('slug', models.SlugField(verbose_name='slug', max_length=255, help_text='Auto-generated. Used in the URL. If changed, the URL will change. Clear it to have the slug re-created.', blank=True)),
                ('master', models.ForeignKey(null=True, related_name='translations', to='aldryn_jobs.JobCategory', editable=False)),
            ],
            options={
                'verbose_name': 'job category Translation',
                'default_permissions': (),
                'db_table': 'aldryn_jobs_jobcategory_translation',
                'db_tablespace': '',
                'managed': True,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='JobListPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, parent_link=True, primary_key=True, serialize=False, to='cms.CMSPlugin')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='JobOpening',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('is_active', models.BooleanField(verbose_name='active?', default=True)),
                ('publication_start', models.DateTimeField(verbose_name='published since', blank=True, null=True)),
                ('publication_end', models.DateTimeField(verbose_name='published until', blank=True, null=True)),
                ('can_apply', models.BooleanField(verbose_name='viewer can apply for the job?', default=True)),
                ('ordering', models.IntegerField(verbose_name='ordering', default=0)),
                ('category', models.ForeignKey(verbose_name='category', related_name='jobs', to='aldryn_jobs.JobCategory')),
                ('content', cms.models.fields.PlaceholderField(slotname='Job Opening Content', null=True, to='cms.Placeholder', editable=False)),
            ],
            options={
                'verbose_name': 'job opening',
                'verbose_name_plural': 'job openings',
                'ordering': ['ordering'],
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='JobOpeningTranslation',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True, serialize=False)),
                ('language_code', models.CharField(db_index=True, verbose_name='Language', max_length=15)),
                ('title', models.CharField(verbose_name='title', max_length=255)),
                ('slug', models.SlugField(db_index=False, verbose_name='slug', max_length=255, help_text='Auto-generated. Used in the URL. If changed, the URL will change. Clear it to have the slug re-created.', blank=True)),
                ('lead_in', djangocms_text_ckeditor.fields.HTMLField(verbose_name='short description', help_text='This text will be displayed in lists.', blank=True)),
                ('master', models.ForeignKey(null=True, related_name='translations', to='aldryn_jobs.JobOpening', editable=False)),
            ],
            options={
                'verbose_name': 'job opening Translation',
                'default_permissions': (),
                'db_table': 'aldryn_jobs_jobopening_translation',
                'db_tablespace': '',
                'managed': True,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='JobsConfig',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True, serialize=False)),
                ('type', models.CharField(verbose_name='type', max_length=100)),
                ('namespace', models.CharField(verbose_name='instance namespace', max_length=100, default=None, unique=True)),
                ('app_data', app_data.fields.AppDataField(editable=False, default='{}')),
                ('placeholder_jobs_detail_bottom', cms.models.fields.PlaceholderField(slotname='jobs_detail_bottom', null=True, related_name='aldryn_jobs_detail_bottom', to='cms.Placeholder', editable=False)),
                ('placeholder_jobs_detail_footer', cms.models.fields.PlaceholderField(slotname='jobs_detail_footer', null=True, related_name='aldryn_jobs_detail_footer', to='cms.Placeholder', editable=False)),
                ('placeholder_jobs_detail_top', cms.models.fields.PlaceholderField(slotname='jobs_detail_top', null=True, related_name='aldryn_jobs_detail_top', to='cms.Placeholder', editable=False)),
                ('placeholder_jobs_list_bottom', cms.models.fields.PlaceholderField(slotname='jobs_list_bottom', null=True, related_name='aldryn_jobs_list_bottom', to='cms.Placeholder', editable=False)),
                ('placeholder_jobs_list_top', cms.models.fields.PlaceholderField(slotname='jobs_list_top', null=True, related_name='aldryn_jobs_list_top', to='cms.Placeholder', editable=False)),
                ('placeholder_jobs_sidebar', cms.models.fields.PlaceholderField(slotname='jobs_sidebar', null=True, related_name='aldryn_jobs_sidebar', to='cms.Placeholder', editable=False)),
                ('placeholder_jobs_top', cms.models.fields.PlaceholderField(slotname='jobs_top', null=True, related_name='aldryn_jobs_top', to='cms.Placeholder', editable=False)),
            ],
            options={
                'verbose_name': 'Aldryn Jobs configuration',
                'verbose_name_plural': 'Aldryn Jobs configurations',
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name='jobopeningtranslation',
            unique_together=set([('language_code', 'master')]),
        ),
        migrations.AddField(
            model_name='joblistplugin',
            name='app_config',
            field=models.ForeignKey(verbose_name='app configuration', help_text='Select appropriate app. configuration for this plugin.', null=True, to='aldryn_jobs.JobsConfig'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='joblistplugin',
            name='jobopenings',
            field=sortedm2m.fields.SortedManyToManyField(verbose_name='job openings', to='aldryn_jobs.JobOpening', help_text='Choose specific Job Openings to show or leave empty to show latest. Note that Job Openings from different app configs will not appear.', blank=True, null=True),
            preserve_default=True,
        ),
        migrations.AlterUniqueTogether(
            name='jobcategorytranslation',
            unique_together=set([('language_code', 'master')]),
        ),
        migrations.AddField(
            model_name='jobcategory',
            name='app_config',
            field=models.ForeignKey(verbose_name='app configuration', null=True, related_name='categories', to='aldryn_jobs.JobsConfig'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='jobcategory',
            name='supervisors',
            field=models.ManyToManyField(verbose_name='supervisors', to=settings.AUTH_USER_MODEL, help_text='Supervisors will be notified via email when a new job application arrives.', blank=True, related_name='job_opening_categories'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='jobcategoriesplugin',
            name='app_config',
            field=models.ForeignKey(verbose_name='app configuration', help_text='Select appropriate app. configuration for this plugin.', null=True, to='aldryn_jobs.JobsConfig'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='jobapplication',
            name='job_opening',
            field=models.ForeignKey(to='aldryn_jobs.JobOpening', related_name='applications'),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import reversion

from django import get_version
from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.urlresolvers import reverse, NoReverseMatch
from django.db import models
from django.db.models.signals import pre_delete
from django.dispatch.dispatcher import receiver
from django.utils.encoding import force_text, python_2_unicode_compatible
from django.utils.timezone import now
from django.utils.translation import ugettext_lazy as _

try:
    from django.utils.importlib import import_module
except ImportError:
    from importlib import import_module

from djangocms_text_ckeditor.fields import HTMLField
from aldryn_reversion.core import version_controlled_content
from aldryn_apphooks_config.managers.parler import (
    AppHookConfigTranslatableManager
)
from aldryn_translation_tools.models import (
    TranslationHelperMixin, TranslatedAutoSlugifyMixin,
)

from cms.models import CMSPlugin
from cms.models.fields import PlaceholderField
from cms.utils.i18n import force_language
from distutils.version import LooseVersion
from functools import partial
from os.path import join as join_path
from parler.models import TranslatableModel, TranslatedFields
from reversion.revisions import default_revision_manager, RegistrationError
from sortedm2m.fields import SortedManyToManyField
from uuid import uuid4

from .cms_appconfig import JobsConfig
from .managers import JobOpeningsManager
from .utils import get_valid_filename

# NOTE: We need to use LooseVersion NOT StrictVersion as Aldryn sometimes uses
# patched versions of Django with version numbers in the form: X.Y.Z.postN
loose_version = LooseVersion(get_version())


def get_user_model_for_fields():
    if loose_version < LooseVersion('1.7.0'):
        return get_user_model()
    else:
        return getattr(settings, 'AUTH_USER_MODEL', 'auth.User')

# We should check if user model is registered, since we're following on that
# relation for EventCoordinator model, if not - register it to
# avoid RegistrationError when registering models that refer to it.
user_model = get_user_model_for_fields()

if loose_version < LooseVersion('1.7.0'):
    # Prior to 1.7 it is pretty straight forward
    revision_manager = reversion.default_revision_manager
    if user_model not in revision_manager.get_registered_models():
        reversion.register(user_model)
else:
    # otherwise it is a pain, but thanks to solution of getting model from
    # https://github.com/django-oscar/django-oscar/commit/c479a1983f326a9b059e157f85c32d06a35728dd  # NOQA
    # we can do almost the same thing from the different side.
    from django.apps import apps
    from django.apps.config import MODELS_MODULE_NAME
    from django.core.exceptions import AppRegistryNotReady

    def get_model(app_label, model_name):
        """
        Fetches a Django model using the app registry.
        This doesn't require that an app with the given app label exists,
        which makes it safe to call when the registry is being populated.
        All other methods to access models might raise an exception about the
        registry not being ready yet.
        Raises LookupError if model isn't found.
        """
        try:
            return apps.get_model(app_label, model_name)
        except AppRegistryNotReady:
            if apps.apps_ready and not apps.models_ready:
                # If this function is called while `apps.populate()` is
                # loading models, ensure that the module that defines the
                # target model has been imported and try looking the model up
                # in the app registry. This effectively emulates
                # `from path.to.app.models import Model` where we use
                # `Model = get_model('app', 'Model')` instead.
                app_config = apps.get_app_config(app_label)
                # `app_config.import_models()` cannot be used here because it
                # would interfere with `apps.populate()`.
                import_module('%s.%s' % (app_config.name, MODELS_MODULE_NAME))
                # In order to account for case-insensitivity of model_name,
                # look up the model through a private API of the app registry.
                return apps.get_registered_model(app_label, model_name)
            else:
                # This must be a different case (e.g. the model really doesn't
                # exist). We just re-raise the exception.
                raise

    # now get the real user model
    model_app_name, model_model = user_model.split('.')
    user_model_object = get_model(model_app_name, model_model)
    # and try to register, if we have a registration error - that means that
    # it has been registered already
    try:
        default_revision_manager.register(user_model_object)
    except RegistrationError:
        pass


def default_jobs_attachment_upload_to(instance, filename):
    date = now().strftime('%Y/%m')
    return join_path(
        'attachments', date, str(uuid4()), get_valid_filename(filename)
    )


jobs_attachment_upload_to = getattr(
    settings, 'ALDRYN_JOBS_ATTACHMENT_UPLOAD_DIR',
    default_jobs_attachment_upload_to
)

jobs_attachment_storage = getattr(
    settings, 'ALDRYN_JOBS_ATTACHMENT_STORAGE', None
)

JobApplicationFileField = partial(
    models.FileField,
    max_length=200,
    blank=True,
    null=True,
    upload_to=jobs_attachment_upload_to,
    storage=jobs_attachment_storage
)


@version_controlled_content(follow=['supervisors', 'app_config'])
@python_2_unicode_compatible
class JobCategory(TranslatedAutoSlugifyMixin,
                  TranslationHelperMixin,
                  TranslatableModel):
    slug_source_field_name = 'name'

    translations = TranslatedFields(
        name=models.CharField(_('name'), max_length=255),
        slug=models.SlugField(_('slug'), max_length=255, blank=True,
            help_text=_('Auto-generated. Used in the URL. If changed, the URL '
                        'will change. Clear it to have the slug re-created.'))
    )

    supervisors = models.ManyToManyField(
        get_user_model_for_fields(), verbose_name=_('supervisors'),
        # FIXME: This is mis-named should be "job_categories"?
        related_name='job_opening_categories',
        help_text=_('Supervisors will be notified via email when a new '
                    'job application arrives.'),
        blank=True
    )
    app_config = models.ForeignKey(JobsConfig, null=True,
        verbose_name=_('app configuration'), related_name='categories')

    ordering = models.IntegerField(_('ordering'), default=0)

    objects = AppHookConfigTranslatableManager()

    class Meta:
        verbose_name = _('job category')
        verbose_name_plural = _('job categories')
        ordering = ['ordering']

    def __str__(self):
        return self.safe_translation_getter('name', str(self.pk))

    def _slug_exists(self, *args, **kwargs):
        """Provide additional filtering for slug generation"""
        qs = kwargs.get('qs', None)
        if qs is None:
            qs = self._get_slug_queryset()
        # limit qs to current app_config only
        kwargs['qs'] = qs.filter(app_config=self.app_config)
        return super(JobCategory, self)._slug_exists(*args, **kwargs)

    def get_absolute_url(self, language=None):
        language = language or self.get_current_language()
        slug = self.safe_translation_getter('slug', language_code=language)
        if self.app_config_id:
            namespace = self.app_config.namespace
        else:
            namespace = 'aldryn_jobs'
        with force_language(language):
            try:
                if not slug:
                    return reverse('{0}:job-opening-list'.format(namespace))
                kwargs = {'category_slug': slug}
                return reverse(
                    '{0}:category-job-opening-list'.format(namespace),
                    kwargs=kwargs,
                    current_app=self.app_config.namespace
                )
            except NoReverseMatch:
                return "/%s/" % language

    def get_notification_emails(self):
        return self.supervisors.values_list('email', flat=True)


@version_controlled_content(follow=['category'])
@python_2_unicode_compatible
class JobOpening(TranslatedAutoSlugifyMixin,
                 TranslationHelperMixin,
                 TranslatableModel):
    slug_source_field_name = 'title'

    translations = TranslatedFields(
        title=models.CharField(_('title'), max_length=255),
        slug=models.SlugField(_('slug'), max_length=255, blank=True,
            unique=False, db_index=False,
            help_text=_('Auto-generated. Used in the URL. If changed, the URL '
                        'will change. Clear it to have the slug re-created.')),
        lead_in=HTMLField(_('short description'), blank=True,
            help_text=_('This text will be displayed in lists.'))
    )

    content = PlaceholderField('Job Opening Content')
    category = models.ForeignKey(JobCategory, verbose_name=_('category'),
        related_name='jobs')
    created = models.DateTimeField(auto_now_add=True)
    is_active = models.BooleanField(_('active?'), default=True)
    publication_start = models.DateTimeField(_('published since'),
        null=True, blank=True)
    publication_end = models.DateTimeField(_('published until'),
        null=True, blank=True)
    can_apply = models.BooleanField(_('viewer can apply for the job?'),
        default=True)

    ordering = models.IntegerField(_('ordering'), default=0)

    objects = JobOpeningsManager()

    class Meta:
        verbose_name = _('job opening')
        verbose_name_plural = _('job openings')
        # DO NOT attempt to add 'translated__title' here.
        ordering = ['ordering', ]

    def __str__(self):
        return self.safe_translation_getter('title', str(self.pk))

    def _slug_exists(self, *args, **kwargs):
        """Provide additional filtering for slug generation"""
        qs = kwargs.get('qs', None)
        if qs is None:
            qs = self._get_slug_queryset()
        # limit qs to current app_config only
        kwargs['qs'] = qs.filter(category__app_config=self.category.app_config)
        return super(JobOpening, self)._slug_exists(*args, **kwargs)

    def get_absolute_url(self, language=None):
        language = language or self.get_current_language()
        slug = self.safe_translation_getter('slug', language_code=language)
        category_slug = self.category.safe_translation_getter(
            'slug', language_code=language
        )
        namespace = getattr(
            self.category.app_config, "namespace", "aldryn_jobs")
        with force_language(language):
            try:
                # FIXME: does not looks correct return category url here
                if not slug:
                    return self.category.get_absolute_url(language=language)
                kwargs = {
                    'category_slug': category_slug,
                    'job_opening_slug': slug,
                }
                return reverse(
                    '{0}:job-opening-detail'.format(namespace),
                    kwargs=kwargs,
                    current_app=self.category.app_config.namespace
                )
            except NoReverseMatch:
                # FIXME: this is wrong, if have some problem in reverse
                #        we should know
                return "/%s/" % language

    def get_active(self):
        return all([
            self.is_active,
            self.publication_start is None or self.publication_start <= now(),
            self.publication_end is None or self.publication_end > now()
        ])

    def get_notification_emails(self):
        return self.category.get_notification_emails()


@version_controlled_content(follow=['job_opening'])
@python_2_unicode_compatible
class JobApplication(models.Model):
    # FIXME: Gender is not the same as salutation.
    MALE = 'male'
    FEMALE = 'female'

    SALUTATION_CHOICES = (
        (MALE, _('Mr.')),
        (FEMALE, _('Mrs.')),
    )

    job_opening = models.ForeignKey(JobOpening, related_name='applications')
    salutation = models.CharField(_('salutation'),
        max_length=20, blank=True, choices=SALUTATION_CHOICES, default=MALE)
    first_name = models.CharField(_('first name'), max_length=20)
    last_name = models.CharField(_('last name'), max_length=20)
    email = models.EmailField(_('email'))
    cover_letter = models.TextField(_('cover letter'), blank=True)
    created = models.DateTimeField(_('created'), auto_now_add=True)
    is_rejected = models.BooleanField(_('rejected?'), default=False)
    rejection_date = models.DateTimeField(_('rejection date'),
        null=True, blank=True)

    class Meta:
        ordering = ['-created']
        verbose_name = _('job application')
        verbose_name_plural = _('job applications')

    def __str__(self):
        return self.get_full_name()

    def get_full_name(self):
        full_name = ' '.join([self.first_name, self.last_name])
        return full_name.strip()


@receiver(pre_delete, sender=JobApplication)
def cleanup_attachments(sender, instance, **kwargs):
    for attachment in instance.attachments.all():
        if attachment:
            attachment.file.delete(False)


@version_controlled_content(follow=['application'])
class JobApplicationAttachment(models.Model):
    application = models.ForeignKey(JobApplication, related_name='attachments',
                                    verbose_name=_('job application'))
    file = JobApplicationFileField()


class BaseJobsPlugin(CMSPlugin):
    app_config = models.ForeignKey(JobsConfig,
        verbose_name=_('app configuration'), null=True,
        help_text=_('Select appropriate app. configuration for this plugin.'))

    class Meta:
        abstract = True


@python_2_unicode_compatible
class JobListPlugin(BaseJobsPlugin):
    """ Store job list for JobListPlugin. """
    jobopenings = SortedManyToManyField(JobOpening, blank=True,
        verbose_name=_('job openings'),
        help_text=_("Choose specific Job Openings to show or leave empty to "
                    "show latest. Note that Job Openings from different "
                    "app configs will not appear."))

    def job_openings(self, namespace):
        """
        Return the selected JobOpening for JobListPlugin.

        If no JobOpening are selected, return all active events for namespace
        and language, sorted by title.
        """
        if self.jobopenings.exists():
            return self.jobopenings.namespace(namespace).active()

        return (
            JobOpening.objects.namespace(self.app_config.namespace)
                              .language(self.language)
                              .active_translations(self.language)
                              .active()
        )

    def __str__(self):
        return force_text(self.pk)

    def copy_relations(self, oldinstance):
        self.jobopenings = oldinstance.jobopenings.all()


class JobCategoriesPlugin(BaseJobsPlugin):

    def __str__(self):
        return _('%s categories') % (self.app_config.get_app_title(), )

    @property
    def categories(self):
        return (
            JobCategory.objects
                       .namespace(self.app_config.namespace)
                       .annotate(count=models.Count('jobs'))
                       .filter(count__gte=1)
                       .order_by('ordering', '-count')
        )



# -*- coding: utf-8 -*-
from south.v2 import SchemaMigration


class Migration(SchemaMigration):

    def forwards(self, orm):
        pass

    def backwards(self, orm):
        pass

    models = {
        'aldryn_jobs.jobcategory': {
            'Meta': {'ordering': "['ordering']", 'object_name': 'JobCategory'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ordering': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        },
        'aldryn_jobs.jobcategorytranslation': {
            'Meta': {'unique_together': "[('language_code', 'master')]", 'object_name': 'JobCategoryTranslation', 'db_table': "'aldryn_jobs_jobcategory_translation'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'null': 'True', 'to': "orm['aldryn_jobs.JobCategory']"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '255', 'blank': 'True'})
        },
        'aldryn_jobs.joboffer': {
            'Meta': {'ordering': "['category__ordering', 'category', '-created']", 'object_name': 'JobOffer'},
            'category': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'jobs'", 'to': "orm['aldryn_jobs.JobCategory']"}),
            'content': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cms.Placeholder']", 'null': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'publication_end': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'publication_start': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'})
        },
        'aldryn_jobs.joboffertranslation': {
            'Meta': {'unique_together': "[('language_code', 'master')]", 'object_name': 'JobOfferTranslation', 'db_table': "'aldryn_jobs_joboffer_translation'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'null': 'True', 'to': "orm['aldryn_jobs.JobOffer']"}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '255', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'cms.placeholder': {
            'Meta': {'object_name': 'Placeholder'},
            'default_width': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'slot': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'})
        }
    }

    complete_apps = ['aldryn_jobs']



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import os
import logging

from django import forms
from django.db.models import Q
from django.conf import settings
from django.core.exceptions import (
    ValidationError,
    ImproperlyConfigured,
)
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext

from aldryn_apphooks_config.utils import setup_config
from app_data import AppDataForm
from cms import __version__ as cms_version
from cms.models import Page
from distutils.version import LooseVersion
from emailit.api import send_mail
from multiupload.fields import MultiFileField
from parler.forms import TranslatableModelForm


from .models import (
    JobApplication, JobApplicationAttachment, JobCategory, JobOpening,
    JobsConfig, JobListPlugin, JobCategoriesPlugin)
from .utils import namespace_is_apphooked

SEND_ATTACHMENTS_WITH_EMAIL = getattr(
    settings, 'ALDRYN_JOBS_SEND_ATTACHMENTS_WITH_EMAIL', True)
DEFAULT_SEND_TO = getattr(settings, 'ALDRYN_JOBS_DEFAULT_SEND_TO', None)

logger = logging.getLogger(__name__)


class AutoAppConfigFormMixin(object):
    """
    If there is only a single AppConfig to choose, automatically select it.
    """
    def __init__(self, *args, **kwargs):
        super(AutoAppConfigFormMixin, self).__init__(*args, **kwargs)
        if 'app_config' in self.fields:
            # if has only one choice, select it by default
            if self.fields['app_config'].queryset.count() == 1:
                self.fields['app_config'].empty_label = None


class JobCategoryAdminForm(AutoAppConfigFormMixin, TranslatableModelForm):

    class Meta:
        model = JobCategory
        fields = ['name', 'slug', 'supervisors', 'app_config']

    def get_app_config_filter(self):
        """
        If there is app_config, returns a filter limiting queryset to
        objects in same app_config, otherwise, returns empty filter.
        """
        if 'app_config' in self.cleaned_data:
            app_config = self.cleaned_data['app_config']
            return Q(app_config=app_config)
        return Q()


class JobOpeningAdminForm(TranslatableModelForm):

    slugified_field = 'title'

    class Meta:
        model = JobOpening
        fields = [
            'title',
            'slug',
            'lead_in',
            'category',
            'is_active',
            'can_apply',
            'publication_start',
            'publication_end'
        ]

        if LooseVersion(cms_version) < LooseVersion('3.0'):
            fields.append('content')

    def __init__(self, *args, **kwargs):
        super(JobOpeningAdminForm, self).__init__(*args, **kwargs)

        # small monkey patch to show better label for categories
        def label_from_instance(category_object):
            return "{0} / {1}".format(
                category_object.app_config,
                category_object
            )
        try:
            self.fields['category'].label_from_instance = label_from_instance
        except KeyError:
            # When the form is invoked by the render_model template tag with a
            # list of explicitly set fields, category might not be present.
            pass

    def get_app_config_filter(self):
        """
        If there is a category, returns a filter limiting the queryset to
        objects in the same app_config, otherwise, returns empty filter.
        """
        if 'category' in self.cleaned_data:
            app_config = self.cleaned_data['category'].app_config
            return Q(category__app_config=app_config)
        return Q()


class JobApplicationForm(forms.ModelForm):
    FIVE_MEGABYTES = 1024 * 1024 * 5
    attachments = MultiFileField(
        max_num=getattr(settings, 'ALDRYN_JOBS_ATTACHMENTS_MAX_COUNT', 5),
        min_num=getattr(settings, 'ALDRYN_JOBS_ATTACHMENTS_MIN_COUNT', 0),
        max_file_size=getattr(
            settings, 'ALDRYN_JOBS_ATTACHMENTS_MAX_FILE_SIZE', FIVE_MEGABYTES),
        required=False
    )

    def __init__(self, *args, **kwargs):
        self.job_opening = kwargs.pop('job_opening')
        if not hasattr(self, 'request') and kwargs.get('request') is not None:
            self.request = kwargs.pop('request')
        super(JobApplicationForm, self).__init__(*args, **kwargs)

    class Meta:
        model = JobApplication
        fields = [
            'salutation',
            'first_name',
            'last_name',
            'email',
            'cover_letter',
        ]

    def save(self, commit=True):
        super(JobApplicationForm, self).save(commit=False)
        self.instance.job_opening = self.job_opening

        if commit:
            self.instance.save()

        for attachment in self.cleaned_data['attachments']:
            att = JobApplicationAttachment(
                application=self.instance, file=attachment)
            att.save()

        # additional actions while applying for the job
        try:
            self.send_confirmation_email()
        except:
            # We're handling ANY exception here because we don't want to
            # prevent the form from ultimately getting saved here.
            logger.exception('Could not send a confirmation email!')

        try:
            self.send_staff_notifications()
        except:
            # We're handling ANY exception here because we don't want to
            # prevent the form from ultimately getting saved here.
            logger.exception('Could not send a staff notifications!')

        return self.instance

    def send_confirmation_email(self):
        context = {'job_application': self.instance}
        send_mail(recipients=[self.instance.email],
                  context=context,
                  template_base='aldryn_jobs/emails/confirmation')

    def send_staff_notifications(self):
        recipients = list(self.instance.job_opening.get_notification_emails())
        if DEFAULT_SEND_TO:
            recipients += [DEFAULT_SEND_TO]

        app_label = self._meta.model._meta.app_label
        try:
            model_name = self._meta.model._meta.model_name
        except AttributeError:
            # Django < 1.9
            model_name = self._meta.model._meta.module_name

        admin_change_form = reverse(
            'admin:{}_{}_change'.format(app_label, model_name),
            args=(self.instance.pk,)
        )

        context = {
            'job_application': self.instance,
        }
        # make admin change form url available
        if hasattr(self, 'request'):
            context['admin_change_form_url'] = self.request.build_absolute_uri(
                admin_change_form)

        kwargs = {}
        if SEND_ATTACHMENTS_WITH_EMAIL:
            attachments = self.instance.attachments.all()
            if attachments:
                kwargs['attachments'] = []
                for attachment in attachments:
                    attachment.file.seek(0)
                    kwargs['attachments'].append(
                        (os.path.split(
                            attachment.file.name)[1], attachment.file.read(),))
        send_mail(recipients=recipients,
                  context=context,
                  template_base='aldryn_jobs/emails/notification', **kwargs)


class JobsConfigForm(AppDataForm):
    pass


class AppConfigPluginFormMixin(object):
    config_model = JobsConfig

    def __init__(self, *args, **kwargs):
        # config_model should be configured before using this mixin
        if self.config_model is None:
            raise ImproperlyConfigured(
                ugettext('Cannot work properly when config class is '
                         'not provided.'))

        super(AppConfigPluginFormMixin, self).__init__(*args, **kwargs)
        # get available event configs, that have the same namespace
        # as pages with namespaces. that will ensure that user wont
        # select config that is not app hooked because that
        # will lead to a 500 error until that config wont be used.
        available_configs = self.config_model.objects.filter(
            namespace__in=Page.objects.exclude(
                application_namespace__isnull=True).values_list(
                'application_namespace', flat=True))

        published_configs_pks = [
            config.pk for config in available_configs
            if namespace_is_apphooked(config.namespace)]

        self.fields['app_config'].queryset = available_configs.filter(
            pk__in=published_configs_pks)
        # inform user that there are not published namespaces
        # which he shouldn't use
        not_published = self.config_model.objects.exclude(
            pk__in=published_configs_pks).values_list(
            'namespace', flat=True)

        # prepare help messages
        msg_not_published = ugettext(
            'Following {0} exists but either pages are not published, or '
            'there is no apphook. To use them - attach them or publish pages '
            'to which they are attached:'.format(self.config_model.__name__))

        not_published_namespaces = '; '.join(not_published)

        additional_message = None
        if not_published.count() > 0:
            # prepare message with list of not published configs.
            additional_message = '{0}\n<br/>{1}'.format(
                msg_not_published, not_published_namespaces)

        # update help text
        if additional_message:
            self.fields['app_config'].help_text += '\n<br/>{0}'.format(
                additional_message)

        # pre select app config if there is only one option
        if self.fields['app_config'].queryset.count() == 1:
                self.fields['app_config'].empty_label = None

    def clean_app_config(self):
        # since namespace is not a unique thing we need to validate it
        # additionally because it is possible that there is a page with same
        # namespace as a jobs config but which is using other app_config, which
        # also would lead to same 500 error. The easiest way is to try to
        # reverse, in case of success that would mean that the app_config is
        # correct and can be used.
        namespace = self.cleaned_data['app_config'].namespace
        if not namespace_is_apphooked(namespace):
            raise ValidationError(
                ugettext(
                    'Seems that selected Job config is not plugged to any '
                    'page, or maybe that page is not published.'
                    'Please select Job config that is being used.'),
                code='invalid')
        return self.cleaned_data['app_config']


class JobListPluginForm(AppConfigPluginFormMixin, forms.ModelForm):
    model = JobListPlugin

    def clean(self):
        data = super(JobListPluginForm, self).clean()
        # save only events for selected app_config
        selected = data.get('jobopenings', [])
        app_config = data.get('app_config')

        new_jobopenings = []
        if app_config is None:
            pass
        else:
            for job in selected:
                if job.category.app_config == app_config:
                    new_jobopenings.append(job)

        data['jobopenings'] = new_jobopenings
        return data


class JobCategoriesListPluginForm(AppConfigPluginFormMixin, forms.ModelForm):
    model = JobCategoriesPlugin

setup_config(JobsConfigForm, JobsConfig)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.conf import settings
from django.template import RequestContext

from aldryn_search.utils import get_index_base, strip_tags

from .models import JobOpening


class JobOpeningsIndex(get_index_base()):
    haystack_use_for_indexing = getattr(settings, "ALDRYN_JOBS_SEARCH", True)

    INDEX_TITLE = True

    def prepare_pub_date(self, obj):
        return obj.publication_start

    def get_title(self, obj):
        return obj.title

    def get_index_kwargs(self, language):
        return {'translations__language_code': language}

    def get_index_queryset(self, language):
        return self.get_model().objects.active()

    def get_model(self):
        return JobOpening

    def get_search_data(self, obj, language, request):
        text_bits = [strip_tags(obj.lead_in)]
        plugins = obj.content.cmsplugin_set.filter(language=language)
        for base_plugin in plugins:
            instance, plugin_type = base_plugin.get_plugin_instance()
            if instance is not None:
                plugin_content = strip_tags(instance.render_plugin(
                    context=RequestContext(request)))
                text_bits.append(plugin_content)

        return ' '.join(text_bits)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from datetime import datetime, timedelta
from copy import deepcopy

from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.auth.hashers import make_password
from django.core.cache import cache
from django.test import TransactionTestCase
from django.utils.timezone import get_current_timezone
from django.utils.translation import override

from cms import api
from cms.utils import get_cms_setting

from ..models import JobsConfig, JobCategory, JobOpening


def tz_datetime(*args, **kwargs):
    """ Return datetime with arguments in UTC timezone """
    return datetime(tzinfo=get_current_timezone(), *args, **kwargs)


class JobsBaseTestCase(TransactionTestCase):

    default_category_values = {
        'en': {
            'name': 'Default category name en',
            'slug': 'default-category-name-en',
        },
        'de': {
            'name': 'Default category name de',
            'slug': 'default-category-name-de',
        },
    }
    default_job_values = {
        'en': {
            'title': 'Default job opening en',
            'slug': 'default-job-opening-en',
            'lead_in': '<p>Default job for default people! <br/>'
                       'Apply now!</p>',
        },
        'de': {
            'title': 'Default job opening de',
            'slug': 'default-job-opening-de',
            'lead_in': '<p>Default job for default people! Now in German! <br/>'
                       'Apply now!</p>',
        },
    }
    default_plugin_content = {
        'en': 'Awesome job details here EN',
        'de': 'Awesome German job details here DE',
    }
    plugin_values_raw = {
        'en': 'English New, revision {0}, content en',
        'de': 'German revision {0}, new content de',
    }
    opening_values_raw = {
        'en': {
            'title': 'Title revision {0} en',
            'slug': 'title-revision-{0}-en',
            'lead_in': '<p>Job revision {0}! EN</p>',
        },
        'de': {
            'title': 'Title revision {0} de',
            'slug': 'title-revision-{0}-de',
            'lead_in': '<p>Job revision {0}! DE</p>',
        },
    }
    category_values_raw = {
        'en': {
            'name': 'Revision {0} category name en',
            'slug': 'revision-{0}-category-name-en',
        },
        'de': {
            'name': 'Revision {0} category name de',
            'slug': 'revision-{0}-category-name-de',
        },
    }
    default_publication_start = {
        'publication_start': datetime.now(tz=get_current_timezone()),
    }
    default_publication_end = {
        'publication_end': datetime.now(
            tz=get_current_timezone()) + timedelta(days=1),
    }
    application_default_values = {
        'first_name': 'Default_first_name',
        'last_name': 'Default_last_name',
        'email': 'example@example.com',
    }
    application_values_raw = {
        'first_name': 'First_name_{0}',
        'last_name': 'Last_name_{0}',
        'email': 'example_{0}@example.com',
    }
    signup_default_values = {
        'recipient': 'initial@example.com',
        'is_verified': False,
        'is_disabled': False,
        'confirmation_key': 'default confirmation key',
    }
    signup_values_raw = {
        'confirmation_key': 'confirmation key {0}{0}{0}',
        'recipient': 'recipient_{0}@example.com'
    }

    def setUp(self):
        super(JobsBaseTestCase, self).setUp()
        self.template = get_cms_setting('TEMPLATES')[0][0]
        self.language = settings.LANGUAGES[0][0]
        self.app_config = JobsConfig.objects.create(
            namespace='jobs_test_namespace'
        )
        self.default_category = self.create_default_job_category(
            translated=True)
        self.root_page = self.create_root_page()
        self.page = self.create_page()

        self.staff_user_password = 'staff_pw'
        self.staff_user = self.create_staff_user(
            'staff', self.staff_user_password)
        self.super_user_password = 'super_pw'
        self.super_user = self.create_super_user(
            'super', self.super_user_password)

    def create_root_page(self):
        root_page = api.create_page(
            'root page', self.template, self.language, published=True)
        api.create_title('de', 'root page de', root_page)
        root_page.publish('en')
        root_page.publish('de')
        return root_page.reload()

    def create_page(self, title=None, slug=None, namespace=None):

        if namespace is None:
            namespace = self.app_config.namespace

        if title is None:
            title = 'Jobs'

        if slug is None:
            slug = 'jobs-app'

        page = api.create_page(
            title='{0} en'.format(title),
            slug='{0}-en'.format(slug),
            template=self.template,
            language=self.language,
            published=True,
            parent=self.root_page,
            apphook='JobsApp',
            apphook_namespace=namespace)
        api.create_title(
            'de', '{0} de'.format(title), page, slug='{0}-de'.format(slug))
        page.publish('en')
        page.publish('de')
        # unfortunately aphook reload doesn't restart server fast,
        # so we do an empty request otherwise tests might
        # fail because it can't get correct url
        with override('en'):
            empty_url = page.get_absolute_url()
        self.client.get(empty_url)
        return page.reload()

    def tearDown(self):
        super(JobsBaseTestCase, self).tearDown()
        app_config = JobsConfig.objects.filter(pk=self.app_config.pk)
        if app_config:
            app_config.get().delete()
        cache.clear()

    def create_user(self, user_name, user_password, is_staff=False,
                    is_superuser=False):
        return User.objects.create(
            username=user_name,
            first_name='{0} first_name'.format(user_name),
            last_name='{0} last_name'.format(user_name),
            password=make_password(user_password),
            is_staff=is_staff,
            is_superuser=is_superuser
        )

    def create_staff_user(self, user_name, user_password):
        staff_user = self.create_user(user_name, user_password, is_staff=True)
        return staff_user

    def create_super_user(self, user_name, user_password):
        super_user = self.create_user(user_name, user_password,
                                      is_superuser=True)
        return super_user

    def create_default_job_category(self, translated=False, config=None):
        # ensure that we always start with english, since it looks
        # like there is some issues with handling active language
        # between tests cases run

        if config is None:
            config = self.app_config

        with override('en'):
            job_category = JobCategory.objects.create(
                app_config=config,
                **self.default_category_values['en'])

        # check if we need a translated job_category
        if translated:
            job_category.create_translation(
                'de', **self.default_category_values['de'])

        return JobCategory.objects.language('en').get(pk=job_category.pk)

    def create_default_job_opening(self, translated=False, category=None):
        # ensure that we always start with english, since it looks
        # like there is some issues with handling active language
        # between tests cases run

        if category is None:
            category = self.default_category

        with override('en'):
            job_opening = JobOpening.objects.create(
                category=category,
                **self.default_job_values['en'])
            api.add_plugin(
                job_opening.content,
                'TextPlugin',
                'en',
                body=self.default_plugin_content['en'])

        # check if we need a translated job opening
        if translated:
            job_opening.create_translation(
                'de', **self.default_job_values['de'])
            with override('de'):
                api.add_plugin(job_opening.content, 'TextPlugin', 'de',
                               body=self.default_plugin_content['de'])

        return JobOpening.objects.language('en').get(pk=job_opening.pk)

    def make_new_values(self, values_dict, replace_with):
        """
        Replace formatting symbol {0} with replace_with param.
        modifies dates by + timedelta(days=int(replace_with))
        Returns new dictionary with same keys and replaced symbols.
        """
        new_dict = {}
        for key, value in values_dict.items():
            if key in ('publication_start', 'publication_end'):
                new_val = value + timedelta(days=replace_with)
            else:
                new_val = value.format(replace_with)
            new_dict[key] = new_val
        return new_dict

    def create_new_job_opening(self, data):
        with override('en'):
            job_opening = JobOpening.objects.create(**data)
        return job_opening

    def prepare_data(self, replace_with=1, category=None, update_date=False):
        values = deepcopy(self.opening_values_raw['en'])
        # if we need to change date to something in future
        # we should do it before call to self.make_new_values
        if update_date:
            values.update(self.default_publication_start)
        # make new values adds timedelta days=number which is passed as a
        # second argument
        values = self.make_new_values(values, replace_with)
        # setup category
        if category is None:
            values['category'] = self.default_category
        else:
            values['category'] = category
        # if date was not updated - use the default one
        if not update_date:
            values.update(self.default_publication_start)
        return values



from django.conf import settings
from django.core.urlresolvers import reverse

from django.utils.translation import override
from parler.utils.context import switch_language

from cms import api
from cms.models import Placeholder
from cms.utils import get_cms_setting
from cms.utils.i18n import force_language
from cms.test_utils.testcases import CMSTestCase

from ..models import JobCategory, JobOpening
from ..cms_appconfig import JobsConfig
from ..utils import namespace_is_apphooked

from .base import JobsBaseTestCase, tz_datetime


class JobsAddTest(JobsBaseTestCase):

    def test_create_job_category(self):
        """
        Check if We can create a new job category.
        """
        category_name = 'Administration'
        new_category = JobCategory.objects.create(name=category_name)
        self.assertEqual(new_category.name, 'Administration')
        self.assertEqual(JobCategory.objects.filter(
            pk=new_category.pk).count(), 1)
        self.assertEqual(
            JobCategory.objects.get(pk=new_category.pk).name, category_name)

    def test_create_job_opening(self):
        """
        Check if We can create a new job opening.
        """
        title = 'Programmer'
        opening = JobOpening.objects.create(
            title=title, category=self.default_category)
        self.assertEqual(opening.title, title)
        self.assertEqual(JobOpening.objects.all()[0], opening)

    def test_category_relation_to_job_opening(self):
        """
        Check if We can access a job opening through a category.
        """
        title = 'Senior'
        opening = JobOpening.objects.create(
            title=title, category=self.default_category)
        self.assertIn(opening, self.default_category.jobs.all())

    def test_add_opening_list_plugin_api(self):
        """
        We add an opening to the Plugin and look it up
        """
        title = 'Manager'
        JobOpening.objects.create(title=title, category=self.default_category)
        placeholder = self.page.placeholders.all()[0]
        api.add_plugin(placeholder, 'JobList', self.language)
        self.page.publish(self.language)
        with override(self.language):
            url = self.page.get_absolute_url()
        response = self.client.get(url)
        self.assertContains(response, title)


class JobsGeneralTests(CMSTestCase):

    def setUp(self):
        # prepare root page
        self.template = get_cms_setting('TEMPLATES')[0][0]
        self.language = settings.LANGUAGES[0][0]
        self.root_page = api.create_page(
            'root page', self.template, self.language, published=True)

        # create translations for all languages
        for language, _ in settings.LANGUAGES[1:]:
            api.create_title(
                language, self.root_page.get_slug(), self.root_page)
            self.root_page.publish(language)

    def test_response_code_if_no_jobs_apphooks_created(self):
        # If is no apphooks for aldryn-jobs check that cms still works
        for language, _ in settings.LANGUAGES:
            with override(language):
                root_url = self.root_page.get_absolute_url()
                response = self.client.get(root_url)
                self.assertEqual(response.status_code, 200)


class JobApphookTest(JobsBaseTestCase):

    def setUp(self):
        super(JobApphookTest, self).setUp()
        self.root_page_urls = self.get_root_page_urls()
        self.apphook_urls = self.get_apphook_urls()

    def get_root_page_urls(self):
        """
        get root page urls for all languages.
        """
        root_page_urls = []
        for language, _ in settings.LANGUAGES:
            root_page_url = self.root_page.get_absolute_url()
            root_page_urls.append(root_page_url)
        return root_page_urls

    def get_apphook_urls(self):
        # use to track apphooked urls, should be different for each language
        # at least because of language code
        apphook_urls = []
        # populate apphook_urls with root page url so that we would be sure
        # that url doesn't leads us to a root page of website.

        for language, _ in settings.LANGUAGES:
            with override(language):
                apphook_url = self.page.get_absolute_url()
                self.assertNotIn(apphook_url, self.root_page_urls)
                apphook_urls.append(apphook_url)
        return apphook_urls

    def get_i18n_urls_for_job(self, job_opening):
        job_urls = []
        for language, _ in settings.LANGUAGES:
            with switch_language(job_opening, language):
                job_url = job_opening.get_absolute_url()
                self.assertNotIn(job_url, job_urls)
                job_urls.append(job_url)
        return job_urls

    def test_app_hooked_page_is_available_on_all_languages(self):
        for apphook_url in self.apphook_urls:
            response = self.client.get(apphook_url)
            self.assertEqual(response.status_code, 200)

    def test_apphooked_pages_are_available_for_super_users(self):
        login_result = self.client.login(
            username=self.super_user, password=self.super_user_password)
        self.assertEqual(login_result, True)

        for apphook_url in self.apphook_urls:
            apphook_url_with_toolbar = '{0}?edit'.format(apphook_url)
            response = self.client.get(apphook_url_with_toolbar)
            self.assertEqual(response.status_code, 200)

    def test_apphooked_pages_are_available_for_jobs_and_categories(self):
        self.create_default_job_opening(translated=True)

        for apphook_url in self.apphook_urls:
            response = self.client.get(apphook_url)
            self.assertEqual(response.status_code, 200)

    def test_apphooked_pages_are_available_for_super_user(self):
        """
        Test apphooked pages are available if there is jobs and categories for
        super user
        """
        self.create_default_job_opening(translated=True)

        login_result = self.client.login(
            username=self.super_user, password=self.super_user_password)
        self.assertEqual(login_result, True)

        for apphook_url in self.apphook_urls:
            apphook_url_with_toolbar = '{0}?edit'.format(apphook_url)
            response = self.client.get(apphook_url_with_toolbar)
            self.assertEqual(response.status_code, 200)

    def test_job_opening_is_accessible(self):
        job_opening = self.create_default_job_opening(translated=True)
        job_urls = self.get_i18n_urls_for_job(job_opening)

        for job_url in job_urls:
            response = self.client.get(job_url)
            self.assertEqual(response.status_code, 200)

    def test_job_opening_is_accessible_by_super_user(self):
        job_opening = self.create_default_job_opening(translated=True)
        job_urls = self.get_i18n_urls_for_job(job_opening)

        login_result = self.client.login(
            username=self.super_user, password=self.super_user_password)
        self.assertEqual(login_result, True)

        for job_url in job_urls:
            response = self.client.get(job_url)
            self.assertEqual(response.status_code, 200)

    def test_job_opening_list_page_contains_correct_detail_urls(self):
        job_opening = self.create_default_job_opening(translated=True)

        for language, _ in settings.LANGUAGES:
            with switch_language(job_opening, language):
                job_url = job_opening.get_absolute_url()
                job_title = job_opening.title
            with override(language):
                opening_list_url = self.page.get_absolute_url()

            response = self.client.get(opening_list_url)
            self.assertContains(response, job_url)
            self.assertContains(response, job_title)

    def test_jobs_config_placeholders_are_usable(self):
        # make sure default config is apphooked to a page.
        # FIXME: until migrations are not enabled and ensured to work with
        # tests - this won't test the default namespace =(
        # though this test is (or at least it should be) pretty useful.
        default_config = JobsConfig.objects.get_or_create(
            namespace='aldryn_jobs')[0]
        if not namespace_is_apphooked(default_config.namespace):
            page = api.create_page(
                title='default jobs config en', template=self.template,
                language='en',
                published=True,
                parent=self.root_page,
                apphook='JobsApp',
                apphook_namespace=default_config.namespace,
                publication_date=tz_datetime(2014, 6, 8)
            )
            api.create_title('de', 'default jobs config en', page)
            page.publish('en')
            page.publish('de')
            page_with_default_config = page.reload()
            # get some time for aldryn-apphook-reload to perform actions
            with force_language('en'):
                default_config_url = page_with_default_config.get_absolute_url()
            self.client.get(default_config_url)

        configs = JobsConfig.objects.all()
        # this would be used to build unique value for text plugin
        # namespace-lang-placeholder_name, which would be searched on the page.
        plugin_content_raw = '{0}-{1}-{2}'
        plugins_content = {}
        for cfg in configs:
            cfg_placehodlers = [field for field in cfg._meta.fields
                                if field.__class__ == Placeholder]
            placeholders_names = [placeholder.name for placeholder in
                                  cfg_placehodlers]
            # make sure that we have an empty list to store plugins content
            plugins_content[cfg] = []
            for placeholder_name in placeholders_names:
                placeholder_instance = getattr(cfg, placeholder_name)
                self.assertNotEqual(type(placeholder_instance), type(None))
                plugin_text = plugin_content_raw.format(
                    cfg.namespace, 'en', placeholder_name)
                api.add_plugin(
                    placeholder_instance, 'TextPlugin', 'en',
                    body=plugin_text)
                # track other namespaces plugin content to check
                # their uniqueness
                plugins_content[cfg].append(plugin_text)

        # test <config> placeholder content if it is attached to a page
        skipped = []
        for cfg in configs:
            if not namespace_is_apphooked(cfg.namespace):
                skipped.append(cfg)
                continue

            with force_language('en'):
                apphook_url = reverse('{0}:job-opening-list'.format(
                    cfg.namespace))
            response = self.client.get(apphook_url)

            # test own content
            for text in plugins_content[cfg]:
                self.assertIn(response, text)

            # make sure other namespace plugins are not leaked
            other_configs = [config for config in plugins_content.keys()
                             if config is not cfg]
            all_other_namespace_plugins_text = [
                text for other_config in other_configs
                for text in plugins_content[other_config]]
            for other_plugins_text in all_other_namespace_plugins_text:
                self.assertNotIn(response, other_plugins_text)
        # make sure that we tested at least one config
        self.assertNotEqual(configs.count(), len(skipped))

    def test_one_active_hook_page_shows_openings_for_its_config_only(self):
        new_config = JobsConfig.objects.create(
            namespace='another_apphook_not_active')

        default_opening = self.create_default_job_opening()

        # default apphooked page url
        with override('en'):
            page_url = self.page.get_absolute_url()
            new_category = JobCategory.objects.create(
                name='Completely different namespace',
                slug='completely-different-namespace',
                app_config=new_config)
            new_opening = self.create_new_job_opening(
                self.prepare_data(1, category=new_category))
            default_opening_url = default_opening.get_absolute_url()

        response = self.client.get(page_url)
        # check same apphook openings and their links are present
        self.assertContains(response, default_opening_url)
        self.assertContains(response, default_opening.title)
        # check other apphook openings and their links are absent
        # doesn't makes sense to test url for new opening
        # since it cannot be properly resolved because of absent apphook
        self.assertNotContains(response, new_opening.title)

    def test_two_active_hook_page_shows_openings_for_its_config_only(self):
        # prepare apphook
        new_config = JobsConfig.objects.create(
            namespace='another_apphook_not_active')
        new_apphook_page = self.create_page(
            title='new apphook', slug='new-apphook',
            namespace=new_config.namespace)
        default_opening = self.create_default_job_opening()

        # default apphooked page url
        with override('en'):
            default_page_url = self.page.get_absolute_url()
            new_apphook_page_url = new_apphook_page.get_absolute_url()
            # create new category and opening
            new_category = JobCategory.objects.create(
                name='Completely different namespace',
                slug='completely-different-namespace',
                app_config=new_config)
            new_opening = self.create_new_job_opening(
                self.prepare_data(1, category=new_category))
            # prepare openings links
            new_opening_url = new_opening.get_absolute_url()
            default_opening_url = default_opening.get_absolute_url()

        # test default page
        response = self.client.get(default_page_url)
        # check same apphook openings and their links are present
        self.assertContains(response, default_opening_url)
        self.assertContains(response, default_opening.title)
        # check other apphook openings and their links are absent
        self.assertNotContains(response, new_opening_url)
        self.assertNotContains(response, new_opening.title)

        # test page with new apphook
        response = self.client.get(new_apphook_page_url)
        # check same apphook openings and their links are present
        self.assertContains(response, new_opening_url)
        self.assertContains(response, new_opening.title)
        # check other apphook openings and their links are absent
        self.assertNotContains(response, default_opening_url)
        self.assertNotContains(response, default_opening.title)

    def test_two_active_hook_pages_same_category_and_opening_names(self):
        # regression for issue with two job configs (different namespace)
        # which have same categories names/slugs and same openings title/slugs
        new_config = JobsConfig.objects.create(
            namespace='another_apphook_to_test')
        new_apphook_page = self.create_page(
            title='new apphook', slug='new-apphook',
            namespace=new_config.namespace)

        default_opening = self.create_default_job_opening(translated=True)

        same_name_category = self.create_default_job_category(
            translated=True, config=new_config)
        same_name_opening = self.create_default_job_opening(
            translated=True, category=same_name_category)

        lead_in_text_raw = '{0} lead in for other config'

        for language in ('de', 'en'):
            # get urls
            default_opening_url = default_opening.get_absolute_url(language)
            same_name_opening_url = same_name_opening.get_absolute_url(language)

            with override(language):
                default_app_url = self.page.get_absolute_url()
                other_app_url = new_apphook_page.get_absolute_url()
            with switch_language(same_name_opening, language):
                # change lead in to be different for other opening
                same_name_opening.lead_in = lead_in_text_raw.format(
                    language)
                same_name_opening.save()

            # urls should be different because namespace is different
            self.assertNotEqual(default_app_url, other_app_url)

            response_default = self.client.get(default_app_url)
            self.assertEqual(response_default.status_code, 200)
            self.assertContains(response_default, default_opening_url)

            response_other = self.client.get(other_app_url)
            self.assertEqual(response_other.status_code, 200)
            self.assertContains(response_other, same_name_opening_url)

            # test translated content, unfortunately we need to explicitly
            # switch language
            with switch_language(default_opening, language):
                self.assertContains(response_default, default_opening.title)
            with switch_language(same_name_opening, language):
                self.assertContains(response_other, same_name_opening.title)

    def test_detail_view_same_category_and_opening_names(self):
        # regression for issue with two job configs (different namespace)
        # which have same categories names/slugs and same openings title/slugs
        new_config = JobsConfig.objects.create(
            namespace='another_apphook_to_test')
        self.create_page(
            title='new apphook', slug='new-apphook',
            namespace=new_config.namespace)

        default_opening = self.create_default_job_opening(translated=True)

        same_name_category = self.create_default_job_category(
            translated=True, config=new_config)
        same_name_opening = self.create_default_job_opening(
            translated=True, category=same_name_category)

        lead_in_text_raw = '{0} lead in for other config'

        for language in ('de', 'en'):
            # get urls
            default_opening_url = default_opening.get_absolute_url(language)
            same_name_opening_url = same_name_opening.get_absolute_url(language)
            with switch_language(same_name_opening, language):
                # change lead in to be different for other opening
                same_name_opening.lead_in = lead_in_text_raw.format(
                    language)
                same_name_opening.save()

            # urls should be different because namespace is different
            self.assertNotEqual(default_opening_url, same_name_opening_url)

            # default one
            response_default = self.client.get(default_opening_url)
            self.assertEqual(response_default.status_code, 200)
            # same named
            response_other = self.client.get(same_name_opening_url)
            self.assertEqual(response_other.status_code, 200)
            # test translatable content
            with switch_language(default_opening, language):
                self.assertContains(response_default, default_opening.title)
                self.assertContains(response_default, default_opening.lead_in)
            with switch_language(same_name_opening, language):
                self.assertContains(response_other, same_name_opening.title)
                self.assertContains(response_other, same_name_opening.lead_in)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.conf.urls import patterns, url

from .views import CategoryJobOpeningList, JobOpeningDetail, JobOpeningList

# default view (root url) which is pointing to ^$ url
DEFAULT_VIEW = 'job-opening-list'

urlpatterns = patterns(
    '',

    url(r'^$', JobOpeningList.as_view(),
        name='job-opening-list'),

    url(r'^(?P<category_slug>\w[-_\w]*)/$',
        CategoryJobOpeningList.as_view(),
        name='category-job-opening-list'),

    url(r'^(?P<category_slug>\w[-_\w]*)/(?P<job_opening_slug>\w[-_\w]*)/$',
        JobOpeningDetail.as_view(),
        name='job-opening-detail'),
)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.core.urlresolvers import NoReverseMatch
from django.utils.translation import ugettext_lazy as _

from cms.menu_bases import CMSAttachMenu
from cms.utils import get_language_from_request
from menus.base import NavigationNode
from menus.menu_pool import menu_pool

from .models import JobCategory
from .models import JobOpening


class JobCategoryMenu(CMSAttachMenu):

    name = _("Job Categories Menu")

    def get_nodes(self, request):
        try:
            app_namespace = self.instance.application_namespace
        except AttributeError:
            app_namespace = None
        language = get_language_from_request(request)
        nodes = []
        categories = (
            JobCategory.objects
                       .namespace(app_namespace)
                       .language(language)
                       .active_translations(language)
        )
        for category in categories:
            try:
                node = NavigationNode(category.name,
                                      category.get_absolute_url(),
                                      category.slug)
                nodes.append(node)
            except NoReverseMatch:
                pass
        return nodes


class JobOpeningMenu(CMSAttachMenu):

    name = _("Job Openings Menu")

    def get_nodes(self, request):

        try:
            app_namespace = self.instance.application_namespace
        except AttributeError:
            app_namespace = None

        current_language = get_language_from_request(request)
        nodes = []
        openings = (
            JobOpening.objects
                      .active()
                      .namespace(app_namespace)
                      .language(current_language)
                      .active_translations(current_language)
        )
        for job_opening in openings:
            try:
                node = NavigationNode(
                    title=job_opening.title,
                    url=job_opening.get_absolute_url(),
                    id=job_opening.pk,
                )
            except NoReverseMatch:
                pass
            else:
                nodes.append(node)
        return nodes


menu_pool.register_menu(JobCategoryMenu)
menu_pool.register_menu(JobOpeningMenu)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django import forms
from django.db import transaction
from django.utils.translation import ugettext_lazy as _, ugettext

from cms.api import add_plugin
from cms.utils import permissions
from cms.utils.conf import get_cms_setting
from cms.wizards.wizard_pool import wizard_pool
from cms.wizards.wizard_base import Wizard
from cms.wizards.forms import BaseFormMixin

from djangocms_text_ckeditor.widgets import TextEditorWidget
from djangocms_text_ckeditor.html import clean_html
from parler.forms import TranslatableModelForm

from reversion.revisions import revision_context_manager

from .cms_appconfig import JobsConfig
from .models import JobCategory, JobOpening
from .utils import namespace_is_apphooked


class ConfigCheckMixin(object):

    def user_has_add_permission(self, user, **kwargs):
        """
        Return True if the current user has permission to add a JobCategory or
        JobOpening (depending on value of `perm_string` class variable.
        :param user: The current user
        :param kwargs: Ignored here
        :return: True if user has add permission, else False
        """
        # No one can create a job object, if there is no app_config yet.
        configs = JobsConfig.objects.all()
        if not configs or not any([namespace_is_apphooked(config.namespace)
                                   for config in configs]):
            return False
        # Ensure user has permission to create job objects.
        if user.is_superuser or user.has_perm(self.perm_string):
            return True

        # By default, no permission.
        return False


class JobCategoryWizard(ConfigCheckMixin, Wizard):
    perm_string = "aldryn_jobs.add_jobcategory"


class JobOpeningWizard(ConfigCheckMixin, Wizard):
    perm_string = "aldryn_jobs.add_jobopening"

    def user_has_add_permission(self, user, **kwargs):
        """
        Return True if the current user has permission to add a JobOpening and
        there is at least one JobCategory.
        :param user: The current user
        :param kwargs: Ignored here
        :return: True if user has add permission, else False
        """
        base_perm = super(JobOpeningWizard, self).user_has_add_permission(
            user, **kwargs)
        return base_perm and JobCategory.objects.exists()


class CreateJobCategoryForm(BaseFormMixin, TranslatableModelForm):
    """
    The ModelForm for the JobCategory wizard. Note that JobCategory has few
    translated fields that we need to access, so, we use TranslatableModelForm
    """

    class Meta:
        model = JobCategory
        fields = ['name', 'slug', 'app_config']

    def __init__(self, **kwargs):
        super(CreateJobCategoryForm, self).__init__(**kwargs)
        # If there's only 1 app_config, don't bother show the
        # app_config choice field, we'll choose the option for the user.
        app_configs = JobsConfig.objects.all()
        # check if app config is apphooked
        app_configs = [app_config
                       for app_config in app_configs
                       if namespace_is_apphooked(app_config.namespace)]
        if len(app_configs) == 1:
            self.fields['app_config'].widget = forms.HiddenInput()
            self.fields['app_config'].initial = app_configs[0].pk

    def save(self, commit=True):
        category = super(CreateJobCategoryForm, self).save(commit=False)

        with transaction.atomic():
            with revision_context_manager.create_revision():
                category.save()
                if self.user:
                    revision_context_manager.set_user(self.user)
                revision_context_manager.set_comment(
                    ugettext("Initial version."))

        return category


class CreateJobOpeningForm(BaseFormMixin, TranslatableModelForm):
    """
    The ModelForm for the Job Opening wizard. Note that JobOpening has few
    translated fields that we need to access, so, we use TranslatableModelForm
    """

    content = forms.CharField(
        label="Content", required=False, widget=TextEditorWidget,
        help_text=_("Optional. If provided, will be added to the main body of "
                    "the Opening content."),
    )

    class Meta:
        model = JobOpening
        fields = ['title', 'category', 'is_active', 'lead_in',
                  'publication_start', 'publication_end',
                  'content', 'can_apply', ]

    def __init__(self, **kwargs):
        super(CreateJobOpeningForm, self).__init__(**kwargs)

        # If there's only 1 category, don't bother show the empty label (choice)
        if JobCategory.objects.count() == 1:
            self.fields['category'].empty_label = None
        self.fields['publication_start'].help_text = _(
            'Date Acceptable Formats: 2015-11-30, 11/30/2015, 11/30/15')
        self.fields['publication_end'].help_text = _(
            'Date Acceptable Formats: 2015-11-30, 11/30/2015, 11/30/15')

    def save(self, commit=True):
        job_opening = super(CreateJobOpeningForm, self).save(commit=False)

        # If 'content' field has value, create a TextPlugin with same and add
        # it to the PlaceholderField
        content = clean_html(self.cleaned_data.get('content', ''), False)
        content_plugin = get_cms_setting('WIZARD_CONTENT_PLUGIN')
        if content and permissions.has_plugin_permission(
                self.user, 'TextPlugin', 'add'):

            # If the job_opening has not been saved, then there will be no
            # Placeholder set-up for this question yet, so, ensure we have saved
            # first.
            if not job_opening.pk:
                job_opening.save()

            if job_opening and job_opening.content:
                plugin_kwargs = {
                    'placeholder': job_opening.content,
                    'plugin_type': content_plugin,
                    'language': self.language_code,
                    get_cms_setting('WIZARD_CONTENT_PLUGIN_BODY'): content,
                }
                add_plugin(**plugin_kwargs)

        with transaction.atomic():
            with revision_context_manager.create_revision():
                job_opening.save()
                if self.user:
                    revision_context_manager.set_user(self.user)
                revision_context_manager.set_comment(
                    ugettext("Initial version."))

        return job_opening


job_category_wizard = JobCategoryWizard(
    title=_(u"New job category"),
    weight=500,
    form=CreateJobCategoryForm,
    description=_(u"Create a new job category.")
)

wizard_pool.register(job_category_wizard)

job_opening_wizard = JobOpeningWizard(
    title=_(u"New job opening"),
    weight=550,
    form=CreateJobOpeningForm,
    description=_(u"Create a new job opening.")
)

wizard_pool.register(job_opening_wizard)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.conf import settings
from django.contrib import admin
from django.db import models
from django.utils.safestring import mark_safe
from django.utils.timezone import now
from django.utils.translation import ugettext_lazy as _

from adminsortable2.admin import SortableAdminMixin
from aldryn_apphooks_config.admin import BaseAppHookConfig
from aldryn_reversion.admin import VersionedPlaceholderAdminMixin
from aldryn_translation_tools.admin import (
    AllTranslationsMixin,
    LinkedRelatedInlineMixin,
)
from cms import __version__ as cms_version
from cms.admin.placeholderadmin import FrontendEditableAdminMixin
from distutils.version import LooseVersion
from emailit.api import send_mail
from parler.admin import TranslatableAdmin


from .forms import JobCategoryAdminForm, JobOpeningAdminForm
from .models import JobApplication, JobCategory, JobOpening, JobsConfig


def _send_rejection_email(modeladmin, request, queryset, lang_code='',
                          delete_application=False):
    # 1. send rejection email - this should be refactored to use djangos "bulk"
    #    mail
    #
    # Info: Using mass rejection on many JobApplications can lead to a timeout,
    # since SMTPs are not known to be fast

    for application in queryset:
        context = {'job_application': application, }
        send_mail(recipients=[application.email], context=context,
                  template_base='aldryn_jobs/emails/rejection_letter',
                  language=lang_code.lower())

    # 2. update status or delete objects
    qs_count = queryset.count()
    if not delete_application:
        queryset.update(is_rejected=True, rejection_date=now())
        success_msg = _("Successfully sent {0} rejection email(s).").format(
            qs_count)
    else:
        queryset.delete()
        success_msg = _("Successfully deleted {0} application(s) and sent "
                        "rejection email.").format(qs_count)

    # 3. inform user with success message
    modeladmin.message_user(request, success_msg)
    return


class SendRejectionEmail(object):
    def __init__(self, lang_code=''):
        super(SendRejectionEmail, self).__init__()
        self.lang_code = lang_code.upper()
        self.name = 'send_rejection_email_{0}'.format(self.lang_code)
        self.title = _("Send rejection e-mail {0}").format(self.lang_code)

    def __call__(self, modeladmin, request, queryset, *args, **kwargs):
        _send_rejection_email(modeladmin, request, queryset,
                              lang_code=self.lang_code)


class SendRejectionEmailAndDelete(SendRejectionEmail):
    def __init__(self, lang_code=''):
        super(SendRejectionEmailAndDelete, self).__init__(lang_code)
        self.name = 'send_rejection_and_delete_{0}'.format(self.lang_code)
        self.title = _("Send rejection e-mail and delete "
                       "application {0}").format(self.lang_code)

    def __call__(self, modeladmin, request, queryset, *args, **kwargs):
        _send_rejection_email(modeladmin, request, queryset,
                              lang_code=self.lang_code, delete_application=True)


class JobApplicationAdmin(VersionedPlaceholderAdminMixin, admin.ModelAdmin):
    list_display = ['__str__', 'job_opening', 'created', 'is_rejected',
                    'rejection_date', ]
    list_filter = ['job_opening', 'is_rejected']
    readonly_fields = ['get_attachment_address']
    raw_id_fields = ['job_opening']

    fieldsets = [
        (_('Job Opening'), {
            'fields': ['job_opening']
        }),
        (_('Personal information'), {
            'fields': ['salutation', 'first_name', 'last_name', 'email']
        }),
        (_('Cover letter & attachments'), {
            'fields': ['cover_letter', 'get_attachment_address']
        })
    ]

    def get_actions(self, request):
        actions = super(JobApplicationAdmin, self).get_actions(request)
        language_codes = [language[0] for language in settings.LANGUAGES]
        for lang_code in language_codes:
            send_rejection_email = SendRejectionEmail(lang_code=lang_code)
            actions[send_rejection_email.name] = (
                send_rejection_email,
                send_rejection_email.name,
                send_rejection_email.title
            )
            send_rejection_and_delete = SendRejectionEmailAndDelete(
                lang_code=lang_code)
            actions[send_rejection_and_delete.name] = (
                send_rejection_and_delete,
                send_rejection_and_delete.name,
                send_rejection_and_delete.title
            )
        return actions

    def has_add_permission(self, request):
        # Don't allow creation of "new" applications via admin-backend until
        # it's properly implemented
        return False

    def get_attachment_address(self, instance):
        attachment_link = '<a href="{address}">{address}</a>'
        attachments = []

        for attachment in instance.attachments.all():
            if attachment:
                attachments.append(
                    attachment_link.format(address=attachment.file.url))
        return mark_safe('<br>'.join(attachments)) if attachments else '-'

    get_attachment_address.allow_tags = True
    get_attachment_address.short_description = _('Attachments')


class JobCategoryAdmin(VersionedPlaceholderAdminMixin,
                       SortableAdminMixin, AllTranslationsMixin,
                       TranslatableAdmin):
    form = JobCategoryAdminForm
    list_display = ['__str__', 'app_config']
    filter_horizontal = ['supervisors']

    def get_fieldsets(self, request, obj=None):
        fieldsets = [
            (_('Translatable fields'), {
                'fields': ['name', 'slug']
            }),
            (_('Supervisors'), {
                'fields': ['supervisors']
            }),
            (_('Options'), {
                'fields': ['app_config']
            })
        ]
        return fieldsets


class JobApplicationInline(LinkedRelatedInlineMixin, admin.TabularInline):
    model = JobApplication
    fields = ['email', 'is_rejected', ]
    readonly_fields = ['email', 'is_rejected', ]

    def has_add_permission(self, request):
        return False


class JobOpeningAdmin(VersionedPlaceholderAdminMixin,
                      AllTranslationsMixin,
                      SortableAdminMixin,
                      FrontendEditableAdminMixin,
                      TranslatableAdmin):
    form = JobOpeningAdminForm
    list_display = ['__str__', 'category', 'num_applications', ]
    frontend_editable_fields = ('title', 'lead_in')
    inlines = [JobApplicationInline, ]

    def get_fieldsets(self, request, obj=None):
        fieldsets = [
            (_('Translatable fields'), {
                'fields': ['title', 'slug', 'lead_in']
            }),
            (_('Options'), {
                'fields': ['category', 'is_active', 'can_apply']
            }),
            (_('Publication period'), {
                'fields': ['publication_start', 'publication_end']
            })
        ]

        if LooseVersion(cms_version) < LooseVersion('3.0'):
            content_fieldset = {
                'classes': ['plugin-holder', 'plugin-holder-nopage'],
                'fields': ['content']
            }
            fieldsets.append((_('Content'), content_fieldset))
        return fieldsets

    def get_queryset(self, request):
        qs = super(JobOpeningAdmin, self).queryset(request)
        qs = qs.annotate(applications_count=models.Count('applications'))
        return qs

    def num_applications(self, obj):
        return obj.applications_count
    num_applications.short_description = '# Applications'
    num_applications.admin_order_field = 'applications_count'


class JobsConfigAdmin(VersionedPlaceholderAdminMixin, BaseAppHookConfig):
    pass


admin.site.register(JobApplication, JobApplicationAdmin)
admin.site.register(JobCategory, JobCategoryAdmin)
admin.site.register(JobOpening, JobOpeningAdmin)
admin.site.register(JobsConfig, JobsConfigAdmin)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.utils.translation import ugettext as _

from aldryn_reversion.core import version_controlled_content
from aldryn_apphooks_config.models import AppHookConfig
from cms.models.fields import PlaceholderField


@version_controlled_content
class JobsConfig(AppHookConfig):
    # Job PHFs
    placeholder_jobs_top = PlaceholderField(
        'jobs_top', related_name='aldryn_jobs_top')

    placeholder_jobs_sidebar = PlaceholderField(
        'jobs_sidebar', related_name='aldryn_jobs_sidebar')

    # Job detail PHFs
    placeholder_jobs_detail_top = PlaceholderField(
        'jobs_detail_top', related_name='aldryn_jobs_detail_top')

    placeholder_jobs_detail_bottom = PlaceholderField(
        'jobs_detail_bottom', related_name='aldryn_jobs_detail_bottom')

    placeholder_jobs_detail_footer = PlaceholderField(
        'jobs_detail_footer', related_name='aldryn_jobs_detail_footer')

    # Jobs list PHFs
    placeholder_jobs_list_top = PlaceholderField(
        'jobs_list_top', related_name='aldryn_jobs_list_top')

    placeholder_jobs_list_bottom = PlaceholderField(
        'jobs_list_bottom', related_name='aldryn_jobs_list_bottom')

    class Meta:
        verbose_name = _('Aldryn Jobs configuration')
        verbose_name_plural = _('Aldryn Jobs configurations')



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.db.models import Q
from django.utils import timezone

from parler.managers import TranslatableManager, TranslatableQuerySet


class JobOpeningsQuerySet(TranslatableQuerySet):
    def active(self):
        now = timezone.now()
        return self.filter(
            Q(publication_start__isnull=True) | Q(publication_start__lte=now),
            Q(publication_end__isnull=True) | Q(publication_end__gt=now),
            is_active=True
        )

    def namespace(self, namespace):
        return self.filter(category__app_config__namespace=namespace)


class JobOpeningsManager(TranslatableManager):
    def get_queryset(self):
        return JobOpeningsQuerySet(self.model, using=self.db)

    get_query_set = get_queryset

    def active(self):
        return self.get_queryset().active()

    def namespace(self, namespace):
        return self.get_query_set().namespace(namespace)
